@@ -1,8 +1,8 @@
 /*
  *   cda - Command-line CD Audio Player
  *
- *   Copyright (C) 1993-2000  Ti Kan
- *   E-mail: ti@amb.org
+ *   Copyright (C) 1993-2001  Ti Kan
+ *   E-mail: xmcd@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
  */
 
 #ifndef LINT
-static char *_visual_c_ident_ = "@(#)visual.c	6.68 00/01/22";
+static char *_visual_c_ident_ = "@(#)visual.c	6.91 01/07/24";
 #endif
 
 #ifndef NOVISUAL
@@ -37,7 +37,7 @@ static char *_visual_c_ident_ = "@(#)visual.c	6.68 00/01/22";
 #include "common_d/util.h"
 #include "common_d/patchlevel.h"
 #include "libdi_d/libdi.h"
-#include "cddb_d/cddb.h"
+#include "cdinfo_d/cdinfo.h"
 
 /* TRUE and FALSE is redefined in curses.h. */
 #undef TRUE
@@ -68,7 +68,7 @@ static char *_visual_c_ident_ = "@(#)visual.c	6.68 00/01/22";
 #include "cda_d/visual.h"
 
 extern appdata_t	app_data;
-extern cddb_incore_t	cur_db;
+extern cdinfo_incore_t	*dbp;
 extern char		*emsgp,
 			**cda_devlist,
 			errmsg[],
@@ -93,7 +93,7 @@ STATIC bool_t		isvisual,		/* Currently in visual mode */
 			refresh_sts,		/* Refresh status line */
 			echflag,		/* Own echo flag */
 			savch_echo,		/* Control echo for savch */
-			loaddb = TRUE;		/* Load CD database */
+			loaddb = TRUE;		/* Load CD information */
 STATIC word32_t		oastat0 = (word32_t)-1,	/* Previous status value */
 			oastat1 = (word32_t)-1;
 STATIC WINDOW		*info_win,		/* Scrolling window for info */
@@ -382,32 +382,34 @@ cda_v_errhandler(int code)
 
 
 /*
- * cda_v_inexact_select
- *	Ask the user to select from a list of inexact CDDB matches.
+ * cda_v_match_select
+ *	Ask the user to select from a list of fuzzy CDDB matches.
  *
  * Args:
- *	matchlist - List of inexact matches
+ *	matchlist - List of fuzzy matches
  *
  * Return:
  *	User selection number, or 0 for no selection.
  */
 STATIC int
-cda_v_inexact_select(cddb_match_t *matchlist)
+cda_v_match_select(cdinfo_match_t *matchlist)
 {
 	int		i,
 			n,
 			x;
-	cddb_match_t	*p;
+	cdinfo_match_t	*p;
 	char		input[64];
 
 	wclear(info_win);
 	wmove(info_win, 0, 0);
 	wprintw(info_win, "\n%s\n%s\n\n",
-		"There were no exact matches in the CD database.",
-		"Close matches found:");
+		"CDDB has found the following potential matches for the album",
+		"that you have.  Please choose an appropriate entry:");
 
 	for (p = matchlist, i = 1; p != NULL; p = p->next, i++)
-		wprintw(info_win, "  %2d. %-73s\n", i, p->dtitle);
+		wprintw(info_win, "  %2d. %.30s / %-40s\n", i,
+			(p->artist == NULL) ? "(unknown artist)" : p->artist,
+			(p->title == NULL) ? "(unknown title)" : p->title);
 
 	wprintw(info_win, "  %2d. None of the above\n\n", i);
 
@@ -441,8 +443,9 @@ cda_v_inexact_select(cddb_match_t *matchlist)
 		return 0;
 
 	cda_clrstatus();
-	wprintw(status_win, "Querying CD database...");
+	wprintw(status_win, "Accessing CDDB...");
 	wrefresh(status_win);
+	(void) sleep(2);
 	return (n);
 }
 
@@ -462,9 +465,7 @@ STATIC int
 cda_v_auth(int retrycnt)
 {
 	int	x;
-	char	*name,
-		*pass,
-		input[64];
+	char	input[64];
 
 	wclear(info_win);
 	wmove(info_win, 0, 0);
@@ -503,6 +504,7 @@ cda_v_auth(int retrycnt)
 	--scroll_length;
 
 	/* Get user name */
+	input[0] = '\0';
 	cda_clrstatus();
 	wprintw(status_win, "Username: ");
 	cda_v_echo(TRUE);
@@ -512,37 +514,24 @@ cda_v_auth(int retrycnt)
 	if (input[0] == '\0')
 		return 0;
 
-	name = (char *) MEM_ALLOC("v_auth_name", strlen(input) + 1);
-	if (name == NULL) {
+	if (!util_newstr(&dbp->proxy_user, input)) {
 		CDA_FATAL(app_data.str_nomemory);
 		return 0;
 	}
-	strcpy(name, input);
-	(void) memset(input, 0, 64);
 
 	/* Get password */
+	input[0] = '\0';
 	cda_clrstatus();
 	wprintw(status_win, "Password: ");
 	cda_v_echo(FALSE);
 	wrefresh(status_win);
 	cda_wgetstr(status_win, input, 60);
-	if (input[0] == '\0')
-		pass = NULL;
-	else {
-		pass = (char *) MEM_ALLOC("v_auth_pass", strlen(input) + 1);
-		if (pass == NULL) {
-			CDA_FATAL(app_data.str_nomemory);
-			return 0;
-		}
-		strcpy(pass, input);
-		(void) memset(input, 0, 64);
+
+	if (!util_newstr(&dbp->proxy_passwd, input)) {
+		CDA_FATAL(app_data.str_nomemory);
+		return 0;
 	}
 
-	cddb_set_auth(name, pass == NULL ? "" : pass);
-
-	MEM_FREE(name);
-	MEM_FREE(pass);
-
 	return 1;
 }
 
@@ -564,7 +553,6 @@ cda_screen(int signo)
 {
 	word32_t	arg[CDA_NARGS],
 			discid,
-			queryid,
 			astat0 = 0,
 			astat1 = 0;
 	int		x,
@@ -579,10 +567,11 @@ cda_screen(int signo)
 			idx,
 			min,
 			sec;
-	char		*p;
+	char		*p,
+			*ctrlver;
 	curstat_t	*s = curstat_addr();
 	bool_t		playing;
-	static bool_t	cddb = FALSE;
+	struct stat	stbuf;
 	static unsigned int odiscnum;
 
 	/* Need to refresh function key labels? */
@@ -668,6 +657,23 @@ cda_screen(int signo)
 					);
 				}
 			}
+			else {
+				CDA_FATAL("cannot open send pipe for writing");
+			}
+
+			/* Check FIFOs */
+			if (fstat(cda_sfd[1], &stbuf) < 0 ||
+			    !S_ISFIFO(stbuf.st_mode)) {
+				CDA_FATAL("Send pipe error: Not a FIFO");
+			}
+			if (fstat(cda_rfd[1], &stbuf) < 0 ||
+			    !S_ISFIFO(stbuf.st_mode)) {
+				CDA_FATAL("Recv pipe error: Not a FIFO");
+			}
+			if (fstat(cda_rfd[0], &stbuf) < 0 ||
+			    !S_ISFIFO(stbuf.st_mode)) {
+				CDA_FATAL("Recv pipe error: Not a FIFO");
+			}
 		}
 
 		if (!stat_on)
@@ -739,21 +745,21 @@ cda_screen(int signo)
 			/* Initialize history list */
 			cda_hist_init(s);
 
-			/* Load CD database entry */
+			/* Load CD information */
 			loaddb = FALSE;
 			cda_clrstatus();
-			wprintw(status_win, "Querying CD database...");
+			wprintw(status_win, "Accessing CDDB...");
 			wrefresh(status_win);
+			(void) sleep(2);
 
 			cda_dbclear(s, TRUE);
-			cur_db.discid = discid;
-			queryid = (arg[8] != 0) ? arg[8] : arg[6];
-			cddb = cda_dbload(
-				queryid,
-				cda_v_inexact_select,
-				cda_v_auth,
-				0
-			);
+			dbp->discid = discid;
+			if (cda_dbload(dbp->discid, cda_v_match_select,
+				       cda_v_auth, 0))
+				s->qmode = QMODE_MATCH;
+			else
+				s->qmode = QMODE_NONE;
+
 			cda_clrstatus();
 		}
 
@@ -1004,7 +1010,7 @@ cda_screen(int signo)
 
 				if (arg[0] != 0) {
 					wprintw(status_win, "Program: ");
-					for (i = 0; i < arg[0]; i++) {
+					for (i = 1; i <= arg[0]; i++) {
 						wprintw(status_win, " %02u",
 							arg[i+1]);
 					}
@@ -1086,14 +1092,15 @@ cda_screen(int signo)
 			wattroff(info_win, A_STANDOUT);
 			wprintw(info_win, " for help.\n\n");
 
-			wprintw(info_win, "CD audio        v%s%s PL%d\n",
-				VERSION, VERSION_EXT, PATCHLEVEL);
+			wprintw(info_win, "CD audio        %s.%s%s PL%d\n",
+				VERSION_MAJ, VERSION_MIN, VERSION_EXT,
+				PATCHLEVEL);
 
 			switch ((int) stat_on) {
 			case TRUE:
 				if (cda_sendcmd(CDA_VERSION, arg, &retcode)) {
 					wprintw(info_win,
-						"CD audio daemon v%s\n",
+						"CD audio daemon %s",
 						(char *) arg);
 					break;
 				}
@@ -1101,14 +1108,13 @@ cda_screen(int signo)
 				/*FALLTHROUGH*/
 
 			case FALSE:
-				(void) sprintf((char *) arg, "%s%s PL%d\n%s",
-					       VERSION, VERSION_EXT,
-					       PATCHLEVEL, di_vers());
+				(void) sprintf((char *) arg,
+					       "%s.%s%s PL%d\n%s",
+					       VERSION_MAJ, VERSION_MIN,
+					       VERSION_EXT, PATCHLEVEL,
+					       di_methodstr());
 				break;
 			}
-			wprintw(info_win, "%s\nURL: %s\nE-mail: %s\n\n%s\n",
-				COPYRIGHT, XMCD_URL, EMAIL, GNU_BANNER);
-
 			wprintw(info_win, "\nDevice: %s\n", app_data.device);
 			if (stat_on) {
 				if (!cda_sendcmd(CDA_DEVICE, arg, &retcode)) {
@@ -1124,6 +1130,17 @@ cda_screen(int signo)
 				}
 				wprintw(info_win, "%s\n", (char *) arg);
 			}
+			wprintw(info_win,
+				"\n%s\nURL: %s\nE-mail: %s\n\n%s\n\n",
+				COPYRIGHT, XMCD_URL, EMAIL, GNU_BANNER);
+
+			ctrlver = cdinfo_cddbctrl_ver();
+			wprintw(info_win, "CDDB%s service%s%s\n%s\n",
+				(cdinfo_cddb_ver() == 2) ?
+					"\262" : " \"classic\"",
+				(ctrlver[0] == '\0') ? "" : ": ",
+				(ctrlver[0] == '\0') ? "\n" : ctrlver,
+				CDDB_BANNER);
 
 			prefresh(info_win, scroll_line, 0, 0, 0,
 				 LINES-8, COLS-1);
@@ -1137,7 +1154,7 @@ cda_screen(int signo)
 		wclear(info_win);
 		wmove(info_win, 0, 0);
 
-		/* Get database entry */
+		/* Get CD information */
 		if (RD_ARG_MODE(astat0) == MOD_PLAY ||
 		    RD_ARG_MODE(astat0) == MOD_PAUSE) {
 			track = RD_ARG_TRK(astat1);
@@ -1159,10 +1176,11 @@ cda_screen(int signo)
 			return;
 		}
 
-		if (!cddb || RD_ARG_MODE(astat0) == MOD_STOP) {
+		if (s->qmode != QMODE_MATCH ||
+		    RD_ARG_MODE(astat0) == MOD_STOP) {
 			int	ntrks;
 
-			/* No database entry */
+			/* No CD information */
 			arg[0] = 0;
 			if (!cda_sendcmd(CDA_TOC, arg, &retcode)) {
 				cda_v_errhandler(retcode);
@@ -1176,14 +1194,17 @@ cda_screen(int signo)
 				return;
 			}
 
-			cur_db.discid = arg[0];
+			dbp->discid = arg[0];
 			ntrks = (int) (arg[0] & 0xff);
 
-			wprintw(info_win, "Disc %u  ID: %s %08x%s",
+			wprintw(info_win, "Disc %u  Genre: %s %s",
 				discnum,
-				(cur_db.category[0] == '\0') ?
-					"(no category)" : cur_db.category,
-				arg[0], (cur_db.extd == NULL) ? "" : " *");
+				(dbp->disc.genre == NULL) ?
+					"(unknown genre)" :
+					cdinfo_genre_name(dbp->disc.genre),
+				(dbp->disc.notes != NULL ||
+				 dbp->disc.credit_list != NULL) ?
+					 "*" : "");
 
 			wmove(info_win, 0, COLS-18);
 			wprintw(info_win, "Press ");
@@ -1192,26 +1213,31 @@ cda_screen(int signo)
 			wattroff(info_win, A_STANDOUT);
 			wprintw(info_win, " for help.\n\n");
 
-			if (cddb && cur_db.dtitle != NULL)
-				wprintw(info_win, "%s\n\n", cur_db.dtitle);
-			else
-				wprintw(info_win, "(unknown disc title)\n\n");
+			wprintw(info_win, "%s%s%s\n\n",
+				(dbp->disc.artist == NULL) ?
+					"" : dbp->disc.artist,
+				(dbp->disc.artist != NULL &&
+					dbp->disc.title != NULL) ? " / " : "",
+				(dbp->disc.title == NULL) ?
+					"(unknown title)" : dbp->disc.title);
 
 			for (i = 0; i < (int) ntrks; i++) {
 				RD_ARG_TOC(arg[i+1], trk, playing, min, sec);
 				wprintw(info_win, "%s%02u %02u:%02u  ",
 					playing ? ">" : " ",
 					trk, min, sec);
-				if (cddb && cur_db.trklist[i] != NULL) {
-					wprintw(info_win, "%s%s\n",
-						cur_db.trklist[i],
-						(cur_db.extt[i] != NULL) ?
-							"*" : "");
+				if (dbp->track[i].title != NULL) {
+				    wprintw(info_win, "%s%s\n",
+					dbp->track[i].title,
+					(dbp->track[i].notes != NULL ||
+					 dbp->track[i].credit_list != NULL) ?
+					    "*" : "");
 				}
 				else {
-					wprintw(info_win, "??%s\n",
-						(cur_db.extt[i] != NULL) ?
-							"*" : "");
+				    wprintw(info_win, "??%s\n",
+					(dbp->track[i].notes != NULL ||
+					 dbp->track[i].credit_list != NULL) ?
+					    "*" : "");
 				}
 			}
 
@@ -1220,23 +1246,24 @@ cda_screen(int signo)
 				min, sec);
 		}
 		else {
-			/* Have database entry */
-			if (cur_db.extd == NULL) {
+			/* Got CD information */
+			if (dbp->disc.notes == NULL) {
 				wprintw(info_win,
-				    "No Extended Information for this CD.\n");
+				    "No Disc Notes for this CD.\n");
 			}
 			else {
-				if (cur_db.dtitle != NULL)
-					wprintw(info_win, "%s\n\n",
-						cur_db.dtitle);
-				else
-					wprintw(info_win,
-						"(unknown disc title)\n\n");
+				wprintw(info_win, "%s%s%s\n\n",
+				    (dbp->disc.artist == NULL) ?
+					"" : dbp->disc.artist,
+				    (dbp->disc.artist != NULL &&
+					dbp->disc.title != NULL) ? " / " : "",
+				    (dbp->disc.title == NULL) ?
+					"(unknown title)" : dbp->disc.title);
 
 				/* Not using wprintw here to avoid a bug
 				 * with very long strings
 				 */
-				p = cur_db.extd;
+				p = dbp->disc.notes;
 				for (; *p != '\0'; p++)
 				       waddch(info_win, *p);
 				waddch(info_win, '\n');
@@ -1249,21 +1276,21 @@ cda_screen(int signo)
 			/* If Play or Pause, display track info */
 			if (RD_ARG_MODE(astat0) == MOD_PLAY ||
 			    RD_ARG_MODE(astat0) == MOD_PAUSE) {
-				if (cur_db.trklist[arg[2]] == NULL) {
+				if (dbp->track[arg[2]].title == NULL) {
 					wprintw(info_win,
 						"(No title for track %02u.)\n",
 						arg[1]);
 				}
 				else {
 					wprintw(info_win, "%s\n",
-						cur_db.trklist[arg[2]]);
+						dbp->track[arg[2]].title);
 
-					if (cur_db.extt[arg[2]] != NULL) {
+					if (dbp->track[arg[2]].notes != NULL) {
 						/* Not using wprintw here
 						 * to avoid a bug with very
 						 * long strings
 						 */
-						p = cur_db.extt[arg[2]];
+						p = dbp->track[arg[2]].notes;
 						waddch(info_win, '\n');
 						for (; *p != '\0'; p++)
 						       waddch(info_win, *p);
@@ -1577,7 +1604,7 @@ cda_v_shufprog(int inp, word32_t arg[])
 				if (p == NULL)
 					break;
 
-				while (*p != '\0' && !isdigit(*p))
+				while (*p != '\0' && !isdigit((int) *p))
 					p++;
 			}
 
@@ -2043,13 +2070,13 @@ cda_v_dirtrk(int inp, word32_t arg[])
 	else {
 		mins = secs = 0;
 		if (p != NULL) {
-			while (*p != '\0' && !isdigit(*p))
+			while (*p != '\0' && !isdigit((int) *p))
 				p++;
 
 			mins = strtol(p, &p, 10);
 
 			if (p != NULL) {
-				while (*p != '\0' && !isdigit(*p))
+				while (*p != '\0' && !isdigit((int) *p))
 					p++;
 
 				secs = strtol(p, &p, 10);
