@@ -6,8 +6,8 @@
  *	libdi - CD Audio Player Device Interface Library
  *
  *
- *   Copyright (C) 1993-2000  Ti Kan
- *   E-mail: ti@amb.org
+ *   Copyright (C) 1993-2001  Ti Kan
+ *   E-mail: xmcd@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -28,7 +28,7 @@
 #define __APPENV_H__
 
 #ifndef LINT
-static char *_appenv_h_ident_ = "@(#)appenv.h	6.139 00/01/19";
+static char *_appenv_h_ident_ = "@(#)appenv.h	7.45 01/07/26";
 #endif
 
 
@@ -81,6 +81,7 @@ static char *_appenv_h_ident_ = "@(#)appenv.h	6.139 00/01/19";
 #include <fcntl.h>
 #include <signal.h>
 #include <time.h>
+#include <sys/time.h>
 #include <sys/wait.h>
 #include <sys/stat.h>
 #include <sys/ioctl.h>
@@ -125,6 +126,7 @@ static char *_appenv_h_ident_ = "@(#)appenv.h	6.139 00/01/19";
 #include <errno.h>
 #include <fcntl.h>
 #include <signal.h>
+#include <sys/time.h>
 #include <string.h>
 #include <utsname.h>
 #include <descrip.h>
@@ -198,6 +200,9 @@ typedef struct {
 /*
  * Define these just in case the OS does not support the POSIX definitions
  */
+#if defined(S_IFIFO) && !defined(S_ISFIFO)
+#define S_ISFIFO(m)	(((m) & S_IFMT) == S_IFIFO)
+#endif
 #ifndef S_ISBLK
 #define S_ISBLK(m)	(((m) & S_IFMT) == S_IFBLK)
 #endif
@@ -293,7 +298,7 @@ typedef int			mode_t;
 #endif
 
 
-#ifdef S_IFLNK
+#if defined(S_IFLNK) && !defined(__VMS)
 #define LSTAT		lstat
 #else
 #define LSTAT		stat
@@ -354,29 +359,33 @@ typedef int			mode_t;
 #ifndef __VMS
 #define TEMP_DIR	"/tmp/.cdaudio"		/* Temporary directory */
 #define SYS_CMCFG_PATH	"%s/config/common.cfg"	/* System common cfg file */
+#define USR_CFG_PATH	"%s/.xmcdcfg"		/* User config dir */
+#define USR_PROG_PATH	"%s/.xmcdcfg/prog"	/* User track prog dir */
 #define USR_CMCFG_PATH	"%s/.xmcdcfg/common.cfg"/* User common cfg file */
 #define SYS_DSCFG_PATH	"%s/config/%s"		/* System devspec cfg file */
 #define USR_DSCFG_PATH	"%s/.xmcdcfg/%s"	/* User devspec cfg file */
 #define USR_VINIT_PATH	"%s/.xmcdcfg/%s"	/* User version init file */
 #define USR_HIST_PATH	"%s/.xmcdcfg/history"	/* User xmcd history file */
 #define USR_RMTLOG_PATH	"%s/.xmcdcfg/remote.log"/* User xmcd rmtlog file */
-#define REL_DBDIR_PATH	"%s/cddb/%s"		/* CDDB rel dir path */
-#define CDDBFILE_PATH	"%s/%08x"		/* CDDB file path */
+#define REL_DBDIR_PATH  "%s/cdinfo/%s"		/* CD info rel dir path */
+#define CDINFOFILE_PATH	"%s/%08x"		/* CD info file path */
 #define HELPFILE_PATH	"%s/help/%s"		/* Help file path */
 #define DOCFILE_PATH	"%s/docs/"		/* Documentation file path */
 #define CONCAT_PATH	"%s/%s"			/* Concatenation of path */
 #else
 #define TEMP_DIR	"SYS$LOGIN:TMP.DIR"	/* Temporary directory */
+#define USR_CFG_PATH	"%s.xmcdcfg]"		/* User config dir */
+#define USR_PROG_PATH	"%s.xmcdcfg.prog]"	/* User track prog dir */
 #define SYS_CMCFG_PATH	"%s]common.cfg"		/* System common cfg file */
 #define USR_CMCFG_PATH	"%s.xmcdcfg]common.cfg"	/* User common cfg file */
-#define SYS_DSCFG_PATH	"%s]device.cfg"		/* System devspec cfg file */
-#define USR_DSCFG_PATH	"%s.xmcdcfg]device.cfg"	/* User devspec cfg file */
+#define SYS_DSCFG_PATH	"%s]%s"			/* System devspec cfg file */
+#define USR_DSCFG_PATH	"%s.xmcdcfg]%s"		/* User devspec cfg file */
 #define USR_VINIT_PATH	"%s.xmcdcfg]%s"		/* User version init file */
 #define USR_HIST_PATH	"%s.xmcdcfg]history"	/* User xmcd history file */
 #define USR_RMTLOG_PATH	"%s.xmcdcfg]remote.log"	/* User xmcd rmtlog file */
-#define REL_DBDIR_PATH	"%s.cddb.%s]"		/* CDDB rel dir path */
-#define CDDBFILE_PATH	"%s%08x."		/* CDDB file path */
-#define HELPFILE_PATH	"%s.help]%s"		/* Help file path */
+#define REL_DBDIR_PATH  "%s.cdinfo.%s]"		/* CD info rel dir path */
+#define CDINFOFILE_PATH	"%s%08x."		/* CD info file path */
+#define HELPFILE_PATH	"%s.help.%s"		/* Help file path */
 #define DOCFILE_PATH	"%s.docs]"		/* Documentation file path */
 #define CONCAT_PATH	"%s%s"			/* Concatenation of path */
 #endif
@@ -398,59 +407,60 @@ typedef struct url_attrib {
 	int		tcnt;			/* Count of %T and %t */
 } url_attrib_t;
 
-/* 
- * Web search engine information 
- */
-typedef struct urltmpl_ent {
-	int		idx;			/* Index number */
-	char		*name;			/* Search engine name */
-	char		*action;		/* Action URL+args */
-	char		*fallback;		/* Fallback URL+args */
-	url_attrib_t	a_attrib;		/* Action URL attributes */
-	url_attrib_t	f_attrib;		/* Fallback URL attributes */
-	void		*aux;			/* Auxiliary pointer */
-	struct urltmpl_ent *next;		/* Link to next entry */
-} urltmpl_ent_t;
-
 
 /*
  * Defines for the type field in trkinfo_t
  */
-#define TYP_AUDIO	1			/* Audio track */
-#define TYP_DATA	2			/* Data track */
+#define TYP_AUDIO		1		/* Audio track */
+#define TYP_DATA		2		/* Data track */
 
 
 /*
  * Defines for the mode field in curstat_t
  */
-#define MOD_NODISC	0			/* No disc loaded */
-#define MOD_PLAY	1			/* Play mode */
-#define MOD_PAUSE	2			/* Pause mode */
-#define MOD_STOP	3			/* Stop mode */
-#define MOD_A		4			/* First half of A->B mode */
-#define MOD_AB		5			/* Running A->B mode */
-#define MOD_SAMPLE	6			/* Sample mode */
-#define MOD_BUSY	7			/* Drive is busy */
+#define MOD_NODISC		0		/* No disc loaded */
+#define MOD_PLAY		1		/* Play mode */
+#define MOD_PAUSE		2		/* Pause mode */
+#define MOD_STOP		3		/* Stop mode */
+#define MOD_SAMPLE		4		/* Sample mode */
+#define MOD_BUSY		5		/* Drive is busy */
+
+
+/*
+ * Defines for the segplay field in curstat_t
+ */
+#define SEGP_NONE		0		/* Normal play state */
+#define SEGP_A			1		/* a->? state */
+#define SEGP_AB			2		/* a->b state */
 
 
 /*
  * Defines for the flags field in curstat_t
  */
-#define STAT_SAVE	0x0001			/* Save CDDB before clear */
-#define STAT_CLEAR	0x0002			/* Clear CDDB after save */
-#define STAT_EJECT	0x0004			/* Eject CD after clear */
-#define STAT_EXIT	0x0008			/* Exit program after clear */
-#define STAT_CHGDISC	0x0010			/* Change disc after clear */
+#define STAT_SUBMIT		0x01		/* Submit info before clear */
+#define STAT_EJECT		0x02		/* Eject CD after clear */
+#define STAT_EXIT		0x04		/* Exit program after clear */
+#define STAT_CHGDISC		0x08		/* Change disc after clear */
 
 
 /*
  * Defines for the time_dpy field in curstat_t
  */
-#define TIMEDPY_MAX_MODES	4		/* Max number of modes */
-#define T_ELAPSED		0		/* Per-track elapsed time */
-#define T_ELAPSED_DISC		1		/* Whole-disc elapsed time */
-#define T_REMAIN_TRACK		2		/* Per-track remaining time */
-#define T_REMAIN_DISC		3		/* Whole-disc remaining time */
+#define TIMEDPY_MAX_MODES	6		/* Max number of modes */
+#define T_ELAPSED_TRACK		0		/* Per-track elapsed time */
+#define T_ELAPSED_SEG		1		/* Per-segment elapsed time */
+#define T_ELAPSED_DISC		2		/* Whole-disc elapsed time */
+#define T_REMAIN_TRACK		3		/* Per-track remaining time */
+#define T_REMAIN_SEG		4		/* Per-segment remaining time */
+#define T_REMAIN_DISC		5		/* Whole-disc remaining time */
+
+/*
+ * Defines for the qmode field in curstat_t
+ */
+#define QMODE_NONE		0		/* No status */
+#define QMODE_WAIT		1		/* Waiting for CD info */
+#define QMODE_MATCH		2		/* CD info match found */
+#define QMODE_ERR		3		/* CD info error */
 
 
 /*
@@ -466,40 +476,53 @@ typedef struct {
 } trkinfo_t;
 
 
+/* CD position information */
+typedef struct {
+	word32_t	addr;
+	byte_t		min;
+	byte_t		sec;
+	byte_t		frame;
+	byte_t		pad;
+} cdpos_t;
+
+
 /*
  * Structure containing current status information
  */
 typedef struct {
 	char		*curdev;		/* Current device */
 	byte_t		mode;			/* Player mode */
-	byte_t		time_dpy;		/* Time display mode */
-	word16_t	flags;			/* State flags */
+	byte_t		segplay;		/* a->b mode state */
+	byte_t		time_dpy;		/* Time display state */
+	byte_t		flags;			/* State flags */
+
 	sword32_t	first_disc;		/* First disc */
 	sword32_t	last_disc;		/* Last disc */
 	sword32_t	cur_disc;		/* Current disc */
 	sword32_t	prev_disc;		/* Previous disc */
 	sword32_t	first_trk;		/* First track */
 	sword32_t	last_trk;		/* Last track */
-	byte_t		tot_trks;		/* Total number of tracks */
-	byte_t		tot_min;		/* Total minutes */
-	byte_t		tot_sec;		/* Total seconds */
-	byte_t		tot_frame;		/* Total frame */
-	word32_t	tot_addr;		/* Total block length */
+	sword32_t	tot_trks;		/* Total number of tracks */
+
 	sword32_t	cur_trk;		/* Current track */
 	sword32_t	cur_idx;		/* Current index */
-	byte_t		cur_tot_min;		/* Current absolute minutes */
-	byte_t		cur_tot_sec;		/* Current absolute seconds */
-	byte_t		cur_tot_frame;		/* Current absolute frame */
-	byte_t		cur_trk_min;		/* Current relative minutes */
-	byte_t		cur_trk_sec;		/* Current relative seconds */
-	byte_t		cur_trk_frame;		/* Current relative frame */
-	byte_t		reserved2[2];		/* reserved */
-	word32_t	cur_tot_addr;		/* Current absolute block */
-	word32_t	cur_trk_addr;		/* Current relative block */
+
+	cdpos_t		discpos_tot;		/* Total CD size */
+	cdpos_t		curpos_tot;		/* Current absolute pos */
+	cdpos_t		curpos_trk;		/* Current trk relative pos */
+	cdpos_t		bp_startpos_tot;	/* Block play start abs pos */
+	cdpos_t		bp_endpos_tot;		/* Block play end abs pos */
+	cdpos_t		bp_startpos_trk;	/* Block play start trk pos */
+	cdpos_t		bp_endpos_trk;		/* Block play end trk pos */
+
+	byte_t		aux[4];			/* Auxiliary */
+
 	word32_t	sav_iaddr;		/* Saved index abs block */
+	word32_t	rptcnt;			/* Repeat iteration count */
+
 	trkinfo_t	trkinfo[MAXTRACK];	/* Per-track information */
 	sword32_t	playorder[MAXTRACK];	/* Prog/Shuf sequence */
-	word32_t	rptcnt;			/* Repeat iteration count */
+
 	bool_t		devlocked;		/* Device locked */
 	bool_t		repeat;			/* Repeat mode */
 	bool_t		shuffle;		/* Shuffle mode */
@@ -507,7 +530,7 @@ typedef struct {
 	bool_t		onetrk_prog;		/* Trk list 1-track program */
 	bool_t		caddy_lock;		/* Caddy lock */
 	bool_t		chgrscan;		/* Changer scan in process */
-	byte_t		cddb;			/* CD Database entry type */
+	byte_t		qmode;			/* CD info query mode */
 	byte_t		prog_tot;		/* Prog/Shuf total tracks */
 	byte_t		prog_cnt;		/* Prog/Shuf track counter */
 	byte_t		level;			/* Current volume level */
@@ -519,34 +542,26 @@ typedef struct {
 } curstat_t;
 
 
-/*
- * Default remote CDDB port number
+/* flags bits that control when to generate local discography file
+ * This is for the app_data.discog_mode parameter
  */
-#define CDDB_PORT		888
+#define DISCOG_GEN_CDINFO	0x1	/* After a successful CD info load */
+#define DISCOG_GEN_WWWWARP	0x2	/* When user invokes local discog */
 
-/*
- * Default network timeout interval
- */
-#define CDDB_TIMEOUT		60
-
-
-/*
- * Default Send CDDB site and command
- */
-#define CDDB_MAILSITE		"cddb-submit@submit.cddb.com"
-#define CDDB_MAILCMD		"mailx -s '%S' %A < %F"
 
 /*
  * Default message strings
  */
 #define MAIN_TITLE		"CD Audio Player"
-#define STR_CDDBLOCAL		"loc-db"
-#define STR_CDDBREMOTE		"rmt-db"
-#define STR_CDDBQUERY		"query"
+#define STR_LOCAL		"local"
+#define STR_CDDB		"CDDB\262"
+#define STR_QUERY		"query"
 #define STR_PROGMODE		"prog"
 #define STR_ELAPSE		"elapse"
+#define STR_ELAPSE_SEG		"e-seg"
 #define STR_ELAPSE_DISC		"e-disc"
 #define STR_REMAIN_TRK		"r-trac"
+#define STR_REMAIN_SEG		"r-seg"
 #define STR_REMAIN_DISC		"r-disc"
 #define STR_PLAY		"play"
 #define STR_PAUSE		"pause"
@@ -556,6 +571,7 @@ typedef struct {
 #define STR_BADOPTS		"The following option is unrecognized"
 #define STR_NODISC		"no disc"
 #define STR_BUSY		"cd busy"
+#define STR_UNKNARTIST		"unknown artist"
 #define STR_UNKNDISC		"unknown disc title"
 #define STR_UNKNTRK		"unknown track title"
 #define STR_DATA		"data"
@@ -569,23 +585,26 @@ typedef struct {
 #define STR_NOMEMORY		"Out of memory!"
 #define STR_TMPDIRERR		"Cannot create or access directory %s!"
 #define STR_LIBDIRERR		\
-"Neither the XMcd.libdir resource nor the XMCD_LIBDIR environment is defined!"
+"Cannot determine the xmcd library directory.\n\
+You cannot run the program without going through a full installation.\n\
+If you downloaded the binary distribution from the official xmcd web site,\n\
+be sure to read the installation procedure displayed on the web.\n\
+If you compiled xmcd/cda from source, be sure to read the INSTALL file for\n\
+installation instructions."
 #define STR_LONGPATHERR		"Path or message too long."
 #define STR_NOMETHOD		"Unsupported deviceInterfaceMethod parameter!"
 #define STR_NOVU		"Unsupported driveVendorCode parameter!"
 #define STR_NOHELP		"The help file on this topic is not installed!"
-#define STR_NOLINK		"There is no likely CDDB entry to link to."
-#define STR_NODB		"No CD database directory."
 #define STR_NOCFG		"Cannot open configuration file \"%s\"."
 #define STR_NOINFO		"No information available."
 #define STR_NOTROM		"Device is not a CD-ROM!"
 #define STR_NOTSCSI2		"Device is not SCSI-II compliant."
-#define STR_SEND		"Mail current CDDB entry to \"%s\"?"
-#define STR_MAILERR		"Mail command failed."
+#define STR_SUBMIT		"Submit current disc information to CDDB?"
+#define STR_SUBMITOK		"Submission to CDDB succeeded."
+#define STR_SUBMITERR		"Submission to CDDB failed."
 #define STR_MODERR		"Binary permissions error."
 #define STR_STATERR		"Cannot stat device \"%s\"."
 #define STR_NODERR		"\"%s\" is not the appropriate device type!"
-#define STR_DBINCMPLERR		"Cannot send: Incomplete CD database entry."
 #define STR_SEQFMTERR		"Program sequence string format error."
 #define STR_INVPGMTRK		"Invalid track(s) deleted from program."
 #define STR_RECOVERR		"Recovering from audio playback error..."
@@ -600,28 +619,16 @@ typedef struct {
 #define STR_LNKERR_SUID		"File not linked:\nCannot setuid to %d."
 #define STR_LNKERR_LINK		"File not linked:\nLink failed."
 #define STR_LNKERR_KILLED	"File not linked:\nChild killed. (signal %d)"
-#define STR_SRCHDB		\
-"Searching CD database for potential links...\n\
-This may take some time."
-#define STR_RMTMATCH		\
-"The CD database server cannot find an exact\n\
-match for this CD, but has located a possible\n\
-entry:\n\n\
-Category: %s\n\
-Title: %s\n\n\
-Is this the correct CD?"
-#define STR_CHGSAVE		\
-"%s\n\n\
-The on-screen CD database information\n\
-has changed and was not saved.\n\
-Do you want to save it now?"
+#define STR_CHGSUBMIT		\
+"%s\n%s\n\n\
+The on-screen CD database information has changed.\n\
+Do you want to submit it to CDDB now?"
 #define STR_DEVLIST_UNDEF	"The deviceList parameter is undefined."
 #define STR_DEVLIST_COUNT	\
 "The number of devices in the deviceList parameter is incorrect."
 #define STR_MEDCHG_NOINIT	\
 "Cannot initialize medium changer device:\n\
 Running as a single-disc player"
-#define STR_PROXYERR		"HTTP proxy server configuration error."
 #define STR_AUTHFAIL		\
 "Proxy authorization failed.\n\
 Do you want to re-enter your user name and password?"
@@ -632,10 +639,6 @@ Do you want to re-enter your user name and password?"
 #define STR_CMDFAIL		"Command failed"
 #define STR_RMT_NOTENB		"Remote control is not enabled"
 #define STR_RMT_NOCMD		"Remote control: No command specified"
-#define STR_BROWSER_START	"Starting web browser..."
-#define STR_BROWSER_FAIL	\
-"Cannot access web browser.\n\
-Check your configuration."
 #ifdef __VMS
 #define STR_APPDEF		\
 "The XMCD.DAT resource file cannot be located, or is the wrong\n\
@@ -644,10 +647,12 @@ appropriate directory in order for xmcd to run.  Please check\n\
 your xmcd installation."
 #else
 #define STR_APPDEF		\
-"The app-defaults/XMcd file cannot be located, or is the wrong\n\
-version.  A correct version of this file must be present in the\n\
-appropriate directory in order for xmcd to run.  Please check\n\
-your xmcd installation."
+"The app-defaults/XMcd file cannot be located, or is the wrong version.\n\
+You cannot run xmcd without going through a full installation.\n\
+If you downloaded the binary distribution from the official xmcd web site,\n\
+be sure to read the installation procedure displayed on the web.\n\
+If you compiled xmcd/cda from source, be sure to read the INSTALL file for\n\
+installation instructions."
 #endif
 #define STR_KPMODEDSBL		\
 "The %s mode is enabled.\n\
@@ -657,10 +662,74 @@ Do you want to proceed?"
 "This will delete all entries of your disc history.\n\
 Do you want to proceed?"
 #define STR_CHGRSCAN		"Scanning CD changer slots..."
-#define STR_LOCALDISCOG		"Local Discography"
-#define STR_FILE		"File"
-#define STR_BROWSE_CATEG	"Browse category"
-#define STR_HOWTO_SETUP		"How to set up"
+#define STR_THE			"The"
+#define STR_NONEOFABOVE		"None of the above"
+#define STR_ERROR		"error"
+#define STR_HANDLEREQ		"You must enter a user handle."
+#define STR_HANDLEERR		\
+"Either you got your password wrong, or someone else\n\
+is already using this handle.\n\n\
+Please re-enter the user handle and/or password,\n\
+or click the \"E-mail to me\" button to have your\n\
+password hint sent to you via e-mail."
+#define STR_PASSWDREQ		"You must enter a password."
+#define STR_PASSWDMATCHERR	"The passwords do not match.  Try again."
+#define STR_MAILINGHINT		"Your password hint is being e-mailed to you."
+#define STR_UNKNHANDLE		\
+"CDDB does not have a record of this user handle.\n\
+Try again."
+#define STR_NOHINT		\
+"You did not specify a hint when you first registered.\n\
+There is nothing to e-mail to you."
+#define STR_NOMAILHINT		\
+"You did not specify a e-mail address when you first registered.\n\
+CDDB cannot e-mail your password hint to you."
+#define STR_HINTERR		"Failed to get password hint."
+#define STR_USERREGFAIL		"User registration failed."
+#define STR_NOWWWWARP		\
+"No wwwWarp menu was set up in the wwwwarp.cfg file."
+#define STR_CANNOTINVOKE	"Cannot invoke \"%s\""
+#define STR_STOPLOAD		"Stop CD information load?"
+#define STR_RELOAD		"Re-load CD information?"
+#define STR_NEEDROLE		"You must select a role."
+#define STR_NEEDROLENAME	"You must enter a name for the specified role."
+#define STR_DUPCREDIT		"This credit is already in the list."
+#define STR_DUPTRKCREDIT	\
+"This credit is a duplicate of a track credit (track %s).\n\
+Since an album credit applies to all tracks, it is\n\
+ambiguous to have a track credit to be the same as\n\
+the album credit."
+#define STR_DUPDISCCREDIT	\
+"This credit is a duplicate of an album credit.\n\
+Since an album credit applies to all tracks, it is\n\
+ambiguous to have a track credit to be the same as\n\
+the album credit."
+#define STR_NOFIRST		\
+"The last name is filled in but the first name is not."
+#define STR_NOFIRSTLAST		\
+"\"The\" is enabled, but there is no text in the first name\n\
+field or the last name field."
+#define STR_ALBUMARTIST		"Album artist"
+#define STR_TRACKARTIST		"Track %d artist"
+#define STR_CREDIT		"Credit"
+#define STR_FNAMEGUIDE \
+"Click mouse button 3 on any field for usage guidelines"
+#define STR_NOCATEG		"You must enter category information."
+#define STR_NONAME		"You must enter a name for the site."
+#define STR_INVALURL		"The URL is not valid."
+#define STR_SEGPOSERR		\
+"The end position must be at least a number of\n\
+frames past the start position."
+#define STR_INCSEGINFO		\
+"Incomplete segment information.\n\
+You must enter a segment name, and the start\n\
+and end point track and frame numbers."
+#define STR_INVSEGINFO		\
+"Invalid segment information.\n\
+Check the start and end point track and frame\n\
+numbers."
+#define STR_DISCARDCHG		"Do you want to discard your changes?"
+
 
 /*
  * Application resource/option data
@@ -680,30 +749,23 @@ typedef struct {
 	int		blinkoff_interval;	/* Display blink off (ms) */
 	bool_t		main_showfocus;		/* Highlight kbd focus? */
 	bool_t		instcmap;		/* Install colormap? */
-	bool_t		*remotemode;		/* Remote ctrl mode */
+	bool_t		remotemode;		/* Remote ctrl mode */
+	bool_t		rsvd;			/* Reserved */
 	char		*remotehost;		/* Remote ctrl client host */
 
 	/*
 	 * Common config parameters
 	 */
 	char		*device;		/* Default CD-ROM device */
-	char		*cddb_path;		/* CDDB paths */
-	char		*cddb_filemode;		/* CDDB file permissions */
-	char		*cddb_mailsite;		/* Send CDDB mail site */
-	char		*cddb_mailcmd;		/* Send CDDB mail command */
+	char		*cdinfo_path;		/* CD info paths */
+	char		*cdinfo_filemode;	/* CD info file permissions */
 	char		*proxy_server;		/* http proxy host:port */
-	char		*browser_dir;		/* Web browser direct cmd */
-	char		*browser_rmt;		/* Web browser remote cmd */
-	char		*site1_url;		/* Web site 1 URL */
-	char		*site2_url;		/* Web site 2 URL */
-	char		*site3_url;		/* Web site 3 URL */
-	char		*discog_url;		/* Local Discographies URL */
-	char		*exclude_words;		/* Keywords to exclude */
 	char		*hist_filemode;		/* History file permissions */
-	int		cddb_port;		/* Remote CDDB default port */
-	int		cddb_timeout;		/* Remote CDDB timeout (secs) */
-	int		cddb_rmtautosave;	/* Remote CDDB auto-save */
-	int		cddb_maxhist;		/* Max history count */
+	char		*exclude_words;		/* Keywords to exclude */
+	int		cache_timeout;		/* Local cache timeout (days) */
+	int		srv_timeout;		/* Service timeout (secs) */
+	int		discog_mode;		/* Local discography mode */
+	int		cdinfo_maxhist;		/* Max history count */
 	int		stat_interval;		/* Status poll interval (ms) */
 	int		ins_interval;		/* Insert poll interval (ms) */
 	int		prev_threshold;		/* Previous track/index
@@ -713,9 +775,14 @@ typedef struct {
 	int		timedpy_mode;		/* Default time display mode */
 	int		tooltip_delay;		/* Tool-tip delay interval */
 	int		tooltip_time;		/* Tool-tip active interval */
+
+	word32_t	debug;			/* Debug output level */
+
 	bool_t		ins_disable;		/* Insert poll disable */
-	bool_t		cddb_rmtdsbl;		/* Remote query disable */
-	bool_t		http_proxy;		/* Using http proxy server */
+	bool_t		cdinfo_inetoffln;	/* Internet offline */
+	bool_t		use_proxy;		/* Using proxy server */
+	bool_t		proxy_auth;		/* Use proxy authorization */
+	bool_t		auto_musicbrowser;	/* Auto CDDB Music Browser */
 	bool_t		scsierr_msg;		/* Print SCSI error msg? */
 	bool_t		sol2_volmgt;		/* Solaris 2.x Vol Mgr */
 	bool_t		write_curfile;		/* Enable curr.XXX output */
@@ -723,7 +790,6 @@ typedef struct {
 	bool_t		histfile_dsbl;		/* Disable history file */
 	bool_t		remote_enb;		/* Enable remote control */
 	bool_t		remote_log;		/* Enable rmtctl logfile */
-	bool_t		debug;			/* Verbose debug output */
 
 	/*
 	 * Device-specific config parameters
@@ -738,6 +804,7 @@ typedef struct {
 	int		chg_method;		/* Medium change method */
 	int		base_scsivol;		/* SCSI volume value base */
 	int		min_playblks;		/* Minimum play blocks */
+	bool_t		scsiverck;		/* SCSI version check */
 	bool_t		play10_supp;		/* Play Audio (10) supported */
 	bool_t		play12_supp;		/* Play Audio (12) supported */
 	bool_t		playmsf_supp;		/* Play Audio MSF supported */
@@ -756,6 +823,7 @@ typedef struct {
 	bool_t		play_notur;		/* No Tst U Rdy when playing */
 	bool_t		toc_lba;		/* Rd TOC in LBA mode */
 	bool_t		subq_lba;		/* Rd Subchan in LBA mode */
+	bool_t		rsvd2;			/* Reserved */
 	int		drv_blksz;		/* Drive native block size */
 	int		spinup_interval;	/* spin up delay (sec) */
 
@@ -781,28 +849,30 @@ typedef struct {
 	bool_t		shuffle_mode;		/* Shuffle enable on startup */
 	bool_t		multi_play;		/* Multi-CD playback */
 	bool_t		reverse;		/* Multi-CD reverse playback */
-	bool_t		internal_spkr;		/* Enable built-in speakers */
+	bool_t		rsvd3[3];		/* Reserved */
 
 	/*
 	 * Various application message strings
 	 */
 	char		*main_title;		/* Main window title */
-	char		*str_cddblocal;		/* loc-db */
-	char		*str_cddbremote;	/* rmt-db */
-	char		*str_cddbquery;		/* query */
+	char		*str_local;		/* local */
+	char		*str_cddb;		/* cddb */
+	char		*str_query;		/* query */
 	char		*str_progmode;		/* prog */
 	char		*str_elapse;		/* elapse */
+	char		*str_elapseseg;		/* e-seg */
 	char		*str_elapsedisc;	/* e-disc */
 	char		*str_remaintrk;		/* r-trac */
+	char		*str_remainseg;		/* r-seg */
 	char		*str_remaindisc;	/* r-disc */
 	char		*str_play;		/* play */
 	char		*str_pause;		/* pause */
 	char		*str_ready;		/* ready */
 	char		*str_sample;		/* sample */
-	char		*str_usage;		/* Usage */
 	char		*str_badopts;		/* Bad command-line options */
 	char		*str_nodisc;		/* No disc */
 	char		*str_busy;		/* Device busy */
+	char		*str_unknartist;	/* unknown artist */
 	char		*str_unkndisc;		/* unknown disc title */
 	char		*str_unkntrk;		/* unknown track title */
 	char		*str_data;		/* Data */
@@ -820,18 +890,17 @@ typedef struct {
 	char		*str_nomethod;		/* Invalid di_method */
 	char		*str_novu;		/* Invalid vendor code */
 	char		*str_nohelp;		/* No help available on item */
-	char		*str_nolink;		/* No likely CDDB link */
 	char		*str_nodb;		/* No CDDB directory */
 	char		*str_nocfg;		/* Can't open config file */
 	char		*str_noinfo;		/* No information avail */
 	char		*str_notrom;		/* Not a CD-ROM device */
 	char		*str_notscsi2;		/* Not SCSI-II compliant */
-	char		*str_send;		/* Send CDDB confirm msg */
-	char		*str_mailerr;		/* Mail command failed */
+	char		*str_submit;		/* Submit CDDB confirm msg */
+	char		*str_submitok;		/* Submit CDDB succeeded */
+	char		*str_submiterr;		/* Submit CDDB failed */
 	char		*str_moderr;		/* Binary perms error */
 	char		*str_staterr;		/* Can't stat device */
 	char		*str_noderr;		/* Not a character device */
-	char		*str_dbincmplerr;	/* Incomplete CDDB entry err */
 	char		*str_seqfmterr;		/* Pgm sequence format err */
 	char		*str_invpgmtrk;		/* Inv program trk deleted */
 	char		*str_recoverr;		/* Recovering audio play err */
@@ -846,13 +915,10 @@ typedef struct {
 	char		*str_lnkerr_suid;	/* File link err: setuid */
 	char		*str_lnkerr_link;	/* File link err: link */
 	char		*str_lnkerr_killed;	/* File link err: child kill */
-	char		*str_srchdb;		/* Link search dialog msg */
-	char		*str_rmtmatch;		/* CDDB match dialog msg */
-	char		*str_chgsave;		/* Save change dialog msg */
+	char		*str_chgsubmit;		/* Submit change dialog msg */
 	char		*str_devlist_undef;	/* deviceList no defined */
-	char		*str_devlist_count;	/* deviceList count incorrect */
-	char		*str_medchg_noinit;	/* Cannot init medium changer */
-	char		*str_proxyerr;		/* Proxy server config error */
+	char		*str_devlist_count;	/* deviceList count wrong */
+	char		*str_medchg_noinit;	/* Cannot init medium chgr */
 	char		*str_authfail;		/* Proxy auth failure */
 	char		*str_noclient;		/* Can't find client */
 	char		*str_unsuppcmd;		/* Unsupp remote command */
@@ -861,56 +927,56 @@ typedef struct {
 	char		*str_cmdfail;		/* Command failed */
 	char		*str_rmt_notenb;	/* Remote ctl not enabled */
 	char		*str_rmt_nocmd;		/* Remote ctl no command */
-	char		*str_browser_start;	/* Starting web browser */
-	char		*str_browser_fail;	/* Cannot start browser */
 	char		*str_appdef;		/* app-defaults file error */
 	char		*str_kpmodedsbl;	/* Keypad mode dsbl prompt */
 	char		*str_dlist_delall;	/* Delete all dlist entries */
 	char		*str_chgrscan;		/* CD Changer scanning */
-	char		*str_localdiscog;	/* Local Discography */
-	char		*str_file;		/* File */
-	char		*str_browse_categ;	/* Browse category */
-	char		*str_howto_setup;	/* How to set up */
+	char		*str_the;		/* The */
+	char		*str_noneofabove;	/* None of the above */
+	char		*str_error;		/* error */
+	char		*str_handlereq;		/* User handle required */
+	char		*str_handleerr;		/* User handle error */
+	char		*str_passwdreq;		/* Password required */
+	char		*str_passwdmatcherr;	/* Passwords do not match */
+	char		*str_mailinghint;	/* Mailing passwd hint */
+	char		*str_unknhandle;	/* Unknown handle */
+	char		*str_nohint;		/* No passwd hint available */
+	char		*str_nomailhint;	/* Cannot email passwd hint */
+	char		*str_hinterr;		/* Cannot get passwd hint */
+	char		*str_userregfail;	/* User registration failed */
+	char		*str_nowwwwarp;		/* No wwwwarp menu defs */
+	char		*str_cannotinvoke;	/* Cannot invoke destination */
+	char		*str_stopload;		/* Stop CD info load prompt */
+	char		*str_reload;		/* CD info reload prompt */
+	char		*str_needrole;		/* Need to select role */
+	char		*str_needrolename;	/* Need name for role */
+	char		*str_dupcredit;		/* Credit already in list */
+	char		*str_duptrkcredit;	/* Credit already in tracks */
+	char		*str_dupdisccredit;	/* Credit already in disc */
+	char		*str_nofirst;		/* Last but no first name */
+	char		*str_nofirstlast;	/* No first or last name */
+	char		*str_albumartist;	/* Album artist */
+	char		*str_trackartist;	/* Track # artist */
+	char		*str_credit;		/* Credit */
+	char		*str_fnameguide;	/* Fullname guidelines */
+	char		*str_nocateg;		/* Category info needed */
+	char		*str_noname;		/* Name needed */
+	char		*str_invalurl;		/* Invalid URL */
+	char		*str_segposerr;		/* Segment start/end error */
+	char		*str_incseginfo;	/* Incomplete segment info */
+	char		*str_invseginfo;	/* Invalid segment info */
+	char		*str_discardchg;	/* Discard change? */
 
 	/*
-	 * Short-cut key definitions
+	 * Short-cut key translations
 	 */
-	char		*mode_key;		/* Basic/Normal mode */
-	char		*lock_key;		/* Lock */
-	char		*repeat_key;		/* Repeat */
-	char		*shuffle_key;		/* Shuffle */
-	char		*eject_key;		/* Eject */
-	char		*quit_key;		/* Quit */
-	char		*dbprog_key;		/* Database/Program popup */
-	char		*wwwwarp_key;		/* wwwWarp/Help popup */
-	char		*options_key;		/* Options */
-	char		*time_key;		/* Time */
-	char		*ab_key;		/* A->B */
-	char		*sample_key;		/* Sample */
-	char		*keypad_key;		/* Keypad popup */
-	char		*playpause_key;		/* Play/Pause */
-	char		*stop_key;		/* Stop */
-	char		*prevdisc_key;		/* Prev Disc */
-	char		*nextdisc_key;		/* Next Disc */
-	char		*prevtrk_key;		/* Prev track */
-	char		*nexttrk_key;		/* Next track */
-	char		*previdx_key;		/* Prev index */
-	char		*nextidx_key;		/* Next index */
-	char		*rew_key;		/* REW */
-	char		*ff_key;		/* FF */
-	char		*keypad0_key;		/* Keypad 0 */
-	char		*keypad1_key;		/* Keypad 1 */
-	char		*keypad2_key;		/* Keypad 2 */
-	char		*keypad3_key;		/* Keypad 3 */
-	char		*keypad4_key;		/* Keypad 4 */
-	char		*keypad5_key;		/* Keypad 5 */
-	char		*keypad6_key;		/* Keypad 6 */
-	char		*keypad7_key;		/* Keypad 7 */
-	char		*keypad8_key;		/* Keypad 8 */
-	char		*keypad9_key;		/* Keypad 9 */
-	char		*keypadclear_key;	/* Keypad Clear */
-	char		*keypadenter_key;	/* Keypad Enter */
-	char		*keypadcancel_key;	/* Keypad Cancel */
+	char		*main_hotkeys;		/* Main window */
+	char		*keypad_hotkeys;	/* Keypad window */
+
+	/*
+	 * Miscellaneous
+	 */
+	void		*aux;			/* Auxiliary */
 } appdata_t;
 
 
