@@ -1,6 +1,6 @@
 #!/bin/sh
 #
-# @(#)install.sh	6.125 00/01/14
+# @(#)install.sh	7.57 01/06/24
 #
 # Script to install the software binary and support files on
 # the target system.
@@ -14,7 +14,6 @@
 #	BATCH_BINDIR		Binary executable installation directory
 #	BATCH_LIBDIR		X11 library directory
 #	BATCH_XMCDLIB		Xmcd support files top level directory
-#	BATCH_CDDBDIR		Local CD database top level directory
 #	BATCH_DISCOGDIR		Local discographies top level directory
 #	BATCH_MANDIR		Manual page top level directory
 #	BATCH_MANSUFFIX		Manual page file suffix
@@ -22,8 +21,8 @@
 #    xmcd  - Motif(tm) CD Audio Player
 #    cda   - Command-line CD Audio Player
 #
-#    Copyright (C) 1993-2000  Ti Kan
-#    E-mail: ti@amb.org
+#    Copyright (C) 1993-2001  Ti Kan
+#    E-mail: xmcd@amb.org
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -44,18 +43,16 @@ OPATH="$PATH"
 PATH=/bin:/usr/bin:/sbin:/usr/sbin:/etc:/usr/local/bin
 export PATH
 
-XMCD_VER=2.6
-DEMODB=4102560a
-XMCD_URL=http://metalab.unc.edu/tkan/xmcd/
+XMCD_VER=3.0
+XMCD_URL=http://www.amb.org/xmcd/
 DIRPERM=755
 SCRPERM=755
 FILEPERM=444
-BINPERM=711
 XBINPERM=4711
+BINPERM=711
+GDIRPERM=777
 OWNER=bin
 GROUP=bin
-BINOWNER=bin
-BINGROUP=bin
 XBINOWNER=root
 XBINGROUP=bin
 ERRFILE=/tmp/xmcd.err
@@ -70,25 +67,49 @@ do_exit()
 {
 	if [ $1 -eq 0 ]
 	then
-	    $ECHO "\nInstallation of xmcd is now complete.  Please read"
-	    $ECHO "$XMCDLIB/docs/README for further information."
-	    if [ -n "$BATCH" ]
-	    then
-		$ECHO "\nBefore using xmcd/cda for the first time, you must"
-		$ECHO "set up the software by running the following program:"
-		$ECHO "\n\t${XMCDLIB}/config/config.sh\n"
-		$ECHO "Xmcd/cda will not work until that is done."
-	    fi
-	    $ECHO "\nYou may also want to set up cron(1M) to run the"
-	    $ECHO "$XMCDLIB/scripts/genidx script periodically (e.g., nightly)"
-	    $ECHO "to re-generate the Local Discography category index pages."
+	    $ECHO "\nInstallation of xmcd is now complete."
 	else
-	    $ECHO "\nErrors have occurred in the installation."
+	    $ECHO "\nERROR(s) have occurred in the installation."
 	    if [ $ERRFILE != /dev/null ]
 	    then
-		$ECHO "See $ERRFILE for an error log."
+		$ECHO "See $ERRFILE for details."
 	    fi
 	fi
+
+	if [ -n "$SHLIBEXISTS" ]
+	then
+	    $ECHO "\nNOTICE: A CDDB shared library is already installed on"
+	    $ECHO "your system ($SHLIBEXISTS) and install did not overwrite"
+	    $ECHO "it.  If your existing CDDB shared library is old,"
+	    $ECHO "please remove it and re-install xmcd again."
+	fi
+
+	if [ "$LDSO_NEEDCONF" -eq 1 ]
+	then
+	    $ECHO "\nNOTICE: You need to do the following to allow xmcd and"
+	    $ECHO "cda to find the CDDB shared library.  Xmcd/cda may not"
+	    $ECHO "work until this is done."
+	    if [ -n "$LDSO_CONF" ]
+	    then
+		$ECHO "  - Make sure that $SHLIBDIR is in your"
+		$ECHO "    $LDSO_CONF file."
+	    fi
+	    if [ -n "$LDCONFIG" ]
+	    then
+		$ECHO "  - Login as root and run $LDCONFIG to activate"
+		$ECHO "    the CDDB shared library."
+	    fi
+	fi
+
+	if [ -n "$BATCH" ]
+	then
+	    $ECHO "\nNOTICE: Before using xmcd/cda for the first time, you"
+	    $ECHO "must set up the software by running the following program:"
+	    $ECHO "\n\t${XMCDLIB}/config/config.sh\n"
+	    $ECHO "Xmcd/cda will not work until that is done."
+	fi
+
+	$ECHO "\nPlease read the $XMCDLIB/docs/README file.\n\n"
 	exit $1
 }
 
@@ -160,18 +181,24 @@ do_link()
 	source=$1
 	target=$2
 
+	rm -f $target
+
 	# Try symlink first
 	ln -s $source $target 2>/dev/null
-	if [ $? != 0 ]
-	then
-		# Use hard link
-		ln $source $target 2>/dev/null
-	fi
 	RETSTAT=$?
-	if [ $RETSTAT != 0 ]
+
+	if [ $RETSTAT -ne 0 -a -f $source ]
 	then
-		log_err -p "Cannot link $source -> $target"
+		# Use hard link if a regular file
+		ln $source $target 2>/dev/null
+		RETSTAT=$?
 	fi
+
+	if [ $RETSTAT -ne 0 ]
+	then
+		log_err -n "Cannot link $source -> $target"
+	fi
+
 	return $RETSTAT
 }
 
@@ -236,11 +263,7 @@ inst_file()
 		fi
 
 		$ECHO "\t$target"
-		if [ -f $target ]
-		then
-			rm -f $target
-		fi
-
+		rm -f $target
 		cp $source $target
 		if [ $? != 0 ]
 		then
@@ -403,6 +426,60 @@ link_dynlibs()
 	done
 }
 
+fix_discog()
+{
+	ogenre=$1
+	genre=$2
+	id=$3
+	file=$DISCOGDIR/$genre/$id/index.html
+	gn=`echo $genre | sed -e 's/_/ /g' -e 's/\// -> /g'`
+	gn_s=`echo $gn | sed 's/\//\\\\\//g'`
+
+	if [ ! -f $file ]
+	then
+		return;
+	fi
+
+	out=1
+	cat $file | while read line
+	do
+	    if [ "$line" = '<LI>Browse category:' ]
+	    then
+		echo "<LI><A HREF=\"../../index.html\">Main index</A>"
+		echo "<LI><A HREF=\"../index.html\">$gn index</A>"
+		out=0
+		continue
+	    elif [ "$line" = '</LI>' -a out -eq 0 ]
+	    then
+		out=1
+		continue
+	    fi
+
+	    if [ $out -eq 1 ]
+	    then
+		if [ "$line" = '</BODY>' ]
+		then
+			echo "<HR>"
+			echo "This directory: <B>$DISCOGDIR/$genre/$id</B>"
+		fi
+		echo "$line"
+	    fi
+	done | \
+	sed -e "s/Generated by xmcd 2\..*$/Generated by xmcd $XMCD_VER/" \
+	    -e 's/Copyright .* Ti Kan/Copyright (C) 1993-2001  Ti Kan/' \
+	    -e 's/Browse .* Xmcd web site/Xmcd official web site/' \
+	    -e 's/Browse .* CDDB web site/CDDB - The #1 music info source/' \
+	    -e 's/Browse music reviews/Yahoo! music reviews/' \
+	    -e 's/How to set up Local/How to use Local/' \
+	    -e "s/$ogenre $id/$gn_s $id/" \
+	    -e "s/$ows_s/$XMCDURL_S/g" \
+		>${file}.new 2>/dev/null
+
+	sed 's/\.\.\/\.\./\.\.\/\.\.\/\.\./g' <${file}.new >$file
+	chmod 666 $file 2>/dev/null
+	rm -f ${file}.new
+}
+
 
 #
 # Main execution starts here
@@ -450,9 +527,15 @@ then
 	ECHO=echo
 fi
 
-# If awk doesn't work well on your system, try changing the
-# following to nawk or gawk.
-AWK=awk
+# Use nawk if available
+if [ -x /bin/nawk -o -x /usr/bin/nawk ]
+then
+	AWK=nawk
+else
+	# If awk doesn't work well on your system and you have gawk,
+	# try changing the following to gawk.
+	AWK=awk
+fi
 
 # Remove old error log file
 ERROR=0
@@ -497,6 +580,8 @@ fi
 LINKLIBS=False
 SHELL=/bin/sh
 SCO=False
+LDSO_NEEDCONF=0
+SHLIBEXISTS=""
 
 if [ "$OS_SYS" = A/UX ]
 then
@@ -511,12 +596,17 @@ elif [ "$OS_SYS" = HP-UX ]
 then
 	# HP-UX
 	SHELL=/bin/ksh
+elif [ "$OS_SYS" = Linux ]
+then
+	# Linux
+	LDSO_CONF=/etc/ld.so.conf
+	LDCONFIG=/sbin/ldconfig
 elif [ "$OS_SYS" = SunOS -o "$OS_SYS" = Solaris ]
 then
 	case "$OS_REL" in
 	4.*)
 		# SunOS 4.x
-		LINKLIBS=True
+		LDCONFIG=/usr/etc/ldconfig
 		;;
 	5.*)
 		# SunOS 5.x
@@ -592,8 +682,9 @@ $ECHO "-------------------------------------------------------"
 $ECHO "\nThis is free software and comes with no warranty."
 $ECHO "See the GNU General Public License in the COPYING file"
 $ECHO "for details."
-$ECHO "\nPlease visit the xmcd web site:"
-$ECHO "\n\t$XMCD_URL"
+$ECHO "\nThis software contains support for the Gracenote"
+$ECHO "CDDB(tm) Disc Recognition Service.  See the CDDB file"
+$ECHO "for information."
 
 # Check privilege
 (id | fgrep 'uid=0(root)') >/dev/null 2>&1
@@ -610,7 +701,6 @@ then
 
 		XBINPERM=711
 		XBINOWNER=_default_
-		BINOWNER=_default_
 		OWNER=_default_
 		GROUP=_default_
 	else
@@ -623,12 +713,7 @@ fi
 # Check existence of binaries
 
 MISSING=
-for i in \
-	xmcd_d/xmcd \
-	cda_d/cda \
-	util_d/cddbcmd \
-	util_d/dp2xmcd.sh \
-	util_d/wm2xmcd
+for i in xmcd_d/xmcd cda_d/cda util_d/gobrowser
 do
 	if [ ! -f $i ]
 	then
@@ -695,8 +780,8 @@ else
 fi
 
 
-$ECHO "\n\nThe X binary directory is where the executable binary files"
-$ECHO "will be installed."
+$ECHO "\n\nThe X binary directory is where the xmcd and cda executable files"
+$ECHO "will be installed.  It should be in each xmcd user's PATH environment."
 
 if [ -n "$BATCH" ]
 then
@@ -710,12 +795,17 @@ else
 	do
 		if get_str "\n  Enter X binary directory\n  [${BINDIR}]:"
 		then
-			if [ -d "$ANS" ]
+			if echo "$ANS" | grep "^/" >/dev/null 2>&1
 			then
-				BINDIR=$ANS
-				break
+				if [ -d "$ANS" ]
+				then
+					BINDIR=$ANS
+					break
+				else
+					$ECHO "  Error: $ANS does not exist."
+				fi
 			else
-				$ECHO "  Error: $ANS does not exist."
+				$ECHO "  Error: You must use an absolute path."
 			fi
 		else
 			break
@@ -771,12 +861,17 @@ else
 	do
 		if get_str "\n  Enter X library directory\n  [${LIBDIR}]:"
 		then
-			if [ -d "$ANS" ]
+			if echo "$ANS" | grep "^/" >/dev/null 2>&1
 			then
-				LIBDIR=$ANS
-				break
+				if [ -d "$ANS" ]
+				then
+					LIBDIR=$ANS
+					break
+				else
+					$ECHO "  Error: $ANS does not exist."
+				fi
 			else
-				$ECHO "  Error: $ANS does not exist."
+				$ECHO "  Error: You must use an absolute path."
 			fi
 		else
 			break
@@ -805,11 +900,13 @@ else
 		if get_str "\n  Enter xmcd library directory\n  [${XMCDLIB}]:"
 		then
 			UDIR="$ANS"
-			if [ -d "$UDIR" ]
+			if echo "$UDIR" | grep "^/" >/dev/null 2>&1
 			then
+			    if [ -d "$UDIR" ]
+			    then
 				XMCDLIB=$UDIR
 				break
-			else
+			    else
 				if get_yn \
 				"  Directory $UDIR does not exist.  Create it"
 				then
@@ -817,8 +914,11 @@ else
 					break
 				else
 					$ECHO \
-					"  ERROR: Cannot install in ${UDIR}."
+					"  Error: Cannot install in ${UDIR}."
 				fi
+			    fi
+			else
+				$ECHO "  Error: You must use an absolute path."
 			fi
 		else
 			break
@@ -827,31 +927,6 @@ else
 fi
 
 
-# Determine CDDBDIR
-
-CDDBDIR=$XMCDLIB/cddb
-
-if [ ! -d $CDDBDIR ]
-then
-	$ECHO "\n\nThe CD database directory is where the local CD information"
-	$ECHO "cache will be maintained."
-
-	if [ -n "$BATCH" ]
-	then
-		if [ -n "$BATCH_CDDBDIR" ]
-		then
-			CDDBDIR=$BATCH_CDDBDIR
-		fi
-		$ECHO "\nUsing $CDDBDIR"
-	else
-		if get_str "\n  Enter CD database directory\n  [${CDDBDIR}]:"
-		then
-			CDDBDIR=$ANS
-		fi
-	fi
-fi
-
-
 # Determine DISCOGDIR
 
 DISCOGDIR=$XMCDLIB/discog
@@ -860,7 +935,7 @@ if [ ! -d $DISCOGDIR ]
 then
 	$ECHO "\n\nThe Local Discographies directory is where documents,"
 	$ECHO "images, sound clips and other files related to each CD may"
-	$ECHO "be stored.  These can then be viewed ot played using the"
+	$ECHO "be stored.  These can then be viewed or played using the"
 	$ECHO "wwwWarp feature of xmcd."
 
 	if [ -n "$BATCH" ]
@@ -1035,8 +1110,9 @@ do
 done
 
 
-# Set architecture-specific binary directory
+# Set architecture-specific binary and library directory
 ARCHBIN="${XMCDLIB}/bin-${OS_SYS_T}-${OS_MACH_T}"
+ARCHLIB="${XMCDLIB}/lib-${OS_SYS_T}-${OS_MACH_T}"
 
 
 # Make all necessary directories
@@ -1048,28 +1124,37 @@ make_dir $XMCDLIB/config $DIRPERM $OWNER $GROUP
 make_dir $XMCDLIB/config/.tbl $DIRPERM $OWNER $GROUP
 make_dir $XMCDLIB/docs $DIRPERM $OWNER $GROUP
 make_dir $XMCDLIB/help $DIRPERM $OWNER $GROUP
+for i in xmcd_d/hlpfiles/[A-Z][a-z]
+do
+	if [ -d $i ]
+	then
+		j=`echo $i | sed 's/xmcd\_d\/hlpfiles\///'`
+		make_dir $XMCDLIB/help/$j $DIRPERM $OWNER $GROUP
+	fi
+done
 make_dir $XMCDLIB/pixmaps $DIRPERM $OWNER $GROUP
 make_dir $XMCDLIB/scripts $DIRPERM $OWNER $GROUP
 make_dir $ARCHBIN $DIRPERM $OWNER $GROUP
-make_dir $CDDBDIR $DIRPERM $OWNER $GROUP
-make_dir $DISCOGDIR $DIRPERM $OWNER $GROUP
-
-if [ "$CDDBDIR" != "$XMCDLIB/cddb" ]
-then
-	ln -s "$CDDBDIR" "$XMCDLIB/cddb"
-	$ECHO "\t$XMCDLIB/cddb"
-fi
-if [ "$DISCOGDIR" != "$XMCDLIB/discog" ]
-then
-	ln -s "$DISCOGDIR" "$XMCDLIB/discog"
-	$ECHO "\t$XMCDLIB/discog"
-fi
+make_dir $ARCHLIB $DIRPERM $OWNER $GROUP
+make_dir $DISCOGDIR $GDIRPERM $OWNER $GROUP
 
 if [ -n "$MANDIR" ]
 then
 	make_dir $MANDIR $DIRPERM $OWNER $GROUP
 fi
 
+if [ "$DISCOGDIR" != "$XMCDLIB/discog" ]
+then
+	do_link "$DISCOGDIR" "$XMCDLIB/discog"
+	$ECHO "\t$XMCDLIB/discog"
+fi
+
+# Rename the cddb directory if it exists
+if [ ! -d $XMCDLIB/cdinfo -a -d $XMCDLIB/cddb ]
+then
+	mv $XMCDLIB/cddb $XMCDLIB/cdinfo
+fi
+
 
 # Install files
 
@@ -1077,47 +1162,58 @@ SHELL_S=`echo $SHELL | sed 's/\//\\\\\//g'`
 BINDIR_S=`echo $BINDIR | sed 's/\//\\\\\//g'`
 LIBDIR_S=`echo $LIBDIR | sed 's/\//\\\\\//g'`
 XMCDLIB_S=`echo $XMCDLIB | sed 's/\//\\\\\//g'`
-CDDBDIR_S=`echo $CDDBDIR | sed 's/\//\\\\\//g'`
 DISCOGDIR_S=`echo $DISCOGDIR | sed 's/\//\\\\\//g'`
+XMCDURL_S=`echo $XMCD_URL | sed 's/\//\\\\\//g'`
 
 $ECHO "\nInstalling xmcd files..."
 
 # Private binaries
+for i in cddbcmd dp2xmcd wm2xmcd
+do
+	# Remove deprecated executables
+	rm -f $ARCHBIN/$i $BINDIR/$i
+done
 inst_file xmcd_d/xmcd $ARCHBIN/xmcd $XBINPERM $XBINOWNER $GROUP
 inst_file cda_d/cda $ARCHBIN/cda $XBINPERM $XBINOWNER $GROUP
-inst_file util_d/cddbcmd $ARCHBIN/cddbcmd $BINPERM $BINOWNER $GROUP
-inst_file util_d/dp2xmcd.sh $ARCHBIN/dp2xmcd $SCRPERM $OWNER $GROUP
-inst_file util_d/wm2xmcd $ARCHBIN/wm2xmcd $BINPERM $BINOWNER $GROUP
+inst_file util_d/gobrowser $ARCHBIN/gobrowser $BINPERM $OWNER $GROUP
+
+rm -f $ARCHBIN/README
+echo "The executable files in this directory are not intended to be
+run directly.  Please use the startup wrapper scripts installed in
+$BINDIR." >$ARCHBIN/README
+do_chown $OWNER $ARCHBIN/README
+do_chgrp $GROUP $ARCHBIN/README
+do_chmod $FILEPERM $ARCHBIN/README
+$ECHO "\t$ARCHBIN/README"
 
 # X resource defaults file
 inst_file xmcd_d/XMcd.ad $XMCDLIB/app-defaults/XMcd $FILEPERM $OWNER $GROUP
 
 # XKeysymDB file
-inst_file xmcd_d/XKeysymDB $XMCDLIB/app-defaults/XKeysymDB $FILEPERM $OWNER $GROUP
+inst_file xmcd_d/XKeysymDB $XMCDLIB/app-defaults/XKeysymDB \
+	$FILEPERM $OWNER $GROUP
 
 # Documentation
 rm -f $XMCDLIB/docs/*
-(cd docs_d ;\
-for i in * ;\
-do \
+for i in docs_d/*
+do
 	case $i in
-	SCCS)
+	*SCCS)
 		;;
-	RCS)
+	*RCS)
 		;;
-	CVS)
+	*CVS)
 		;;
 	*)
-		inst_file $i $XMCDLIB/docs/$i $FILEPERM $OWNER $GROUP ;\
+		inst_file $i $XMCDLIB/docs/`basename $i` \
+			$FILEPERM $OWNER $GROUP\
 		;;
 	esac
-done)
+done
 
 # Help files
-rm -f $XMCDLIB/help/*
-for i in xmcd_d/hlpfiles/*.btn xmcd_d/hlpfiles/*.lbl xmcd_d/hlpfiles/*.txw \
-	 xmcd_d/hlpfiles/*.scl xmcd_d/hlpfiles/*.lsw xmcd_d/hlpfiles/*.rbx \
-	 xmcd_d/hlpfiles/*.cbx xmcd_d/hlpfiles/*.opt
+rm -f $XMCDLIB/help/[A-Z]*.??? $XMCDLIB/help/[A-Z][a-z]/[A-Z]*.???
+for i in xmcd_d/hlpfiles/[A-Z][a-z]/*.???
 do
 	j=`echo $i | sed 's/xmcd\_d\/hlpfiles\///'`
 	inst_file $i $XMCDLIB/help/$j $FILEPERM $OWNER $GROUP
@@ -1130,11 +1226,20 @@ do
 done
 
 # Configuration files
+if [ -f $XMCDLIB/config/common.cfg ]
+then
+	diff $XMCDLIB/config/common.cfg libdi_d/common.cfg >/dev/null 2>&1
+	if [ $? -ne 0 ]
+	then
+		# Save old config file for use by config.sh
+		mv $XMCDLIB/config/common.cfg $XMCDLIB/config/common.cfg.old
+	fi
+fi
 inst_file libdi_d/common.cfg $XMCDLIB/config/common.cfg \
 	$FILEPERM $OWNER $GROUP
 inst_file libdi_d/device.cfg $XMCDLIB/config/device.cfg \
 	$FILEPERM $OWNER $GROUP
-inst_file misc_d/sites $XMCDLIB/config/sites \
+inst_file cdinfo_d/wwwwarp.cfg $XMCDLIB/config/wwwwarp.cfg \
 	$FILEPERM $OWNER $GROUP
 
 rm -f $XMCDLIB/config/.tbl/*
@@ -1154,8 +1259,8 @@ sed -e "s/^#!\/bin\/sh.*/#!$SHELL_S/" \
     -e "s/^BINDIR=.*/BINDIR=$BINDIR_S/" \
     -e "s/^LIBDIR=.*/LIBDIR=$LIBDIR_S/" \
     -e "s/^XMCDLIB=.*/XMCDLIB=$XMCDLIB_S/" \
-    -e "s/^CDDBDIR=.*/CDDBDIR=$CDDBDIR_S/" \
     -e "s/^DISCOGDIR=.*/DISCOGDIR=$DISCOGDIR_S/" \
+    -e "s/^XMCD_URL=.*/XMCD_URL=$XMCDURL_S/" \
     -e "s/^OWNER=.*/OWNER=$OWNER/" \
     -e "s/^GROUP=.*/GROUP=$GROUP/" \
     <libdi_d/config.sh >/tmp/xmcdcfg.$$
@@ -1176,15 +1281,18 @@ then
 fi
 
 # Shell scripts
-inst_file misc_d/ncsarmt.sh $XMCDLIB/scripts/ncsarmt $SCRPERM $OWNER $GROUP
-inst_file misc_d/ncsawrap.sh $XMCDLIB/scripts/ncsawrap $SCRPERM $OWNER $GROUP
-inst_file misc_d/nswrap.sh $XMCDLIB/scripts/nswrap $SCRPERM $OWNER $GROUP
+for i in ncsarmt ncsawrap nswrap gobrowser
+do
+	# Remove deprecated scripts
+	rm -f $XMCDLIB/scripts/$i
+done
+
 sed -e "s/^#!\/bin\/sh.*/#!$SHELL_S/" \
     -e "s/^BINDIR=.*/BINDIR=$BINDIR_S/" \
     -e "s/^LIBDIR=.*/LIBDIR=$LIBDIR_S/" \
     -e "s/^XMCDLIB=.*/XMCDLIB=$XMCDLIB_S/" \
-    -e "s/^CDDBDIR=.*/CDDBDIR=$CDDBDIR_S/" \
     -e "s/^DISCOGDIR=.*/DISCOGDIR=$DISCOGDIR_S/" \
+    -e "s/^XMCD_URL=.*/XMCD_URL=$XMCDURL_S/" \
     -e "s/^OWNER=.*/OWNER=$OWNER/" \
     -e "s/^GROUP=.*/GROUP=$GROUP/" \
     <misc_d/genidx.sh >$XMCDLIB/scripts/genidx
@@ -1201,26 +1309,33 @@ sed -e "s/\$SHELL/$SHELL_S/g" \
     -e "s/\$BINDIR/$BINDIR_S/g" \
     -e "s/\$LIBDIR/$LIBDIR_S/g" \
     -e "s/\$XMCDLIB/$XMCDLIB_S/g" \
-    -e "s/\$CDDBDIR/$CDDBDIR_S/g" \
     -e "s/file:\/\/localhost\/\$DISCOGDIR/file:\/\/localhost$DISCOGDIR_S/g" \
     -e "s/\$DISCOGDIR/$DISCOGDIR_S/g" \
+    -e "s/\$XMCD_URL/$XMCDURL_S/g" \
     <misc_d/discog.htm >$DISCOGDIR/discog.html
 do_chown $OWNER $DISCOGDIR/discog.html
 do_chgrp $GROUP $DISCOGDIR/discog.html
 do_chmod $FILEPERM $DISCOGDIR/discog.html
 $ECHO "\t$DISCOGDIR/discog.html"
+for i in xmcd cda
+do
+	# HTML man pages
+	rm -f $DISCOGDIR/${i}.html
+	inst_file ${i}_d/${i}.htm $DISCOGDIR/${i}.html $FILEPERM $OWNER $GROUP
+done
 
 # Startup wrapper script
 rm -f $BINDIR/$STARTUP_SCRIPT
-sed -e "s/XMCD_LIBDIR=.*/XMCD_LIBDIR=$XMCDLIB_S/" \
+sed -e "s/^#!\/bin\/sh.*/#!$SHELL_S/" \
+    -e "s/XMCD_LIBDIR=.*/XMCD_LIBDIR=$XMCDLIB_S/" \
 	misc_d/start.sh >$BINDIR/$STARTUP_SCRIPT
-do_chown $BINOWNER $BINDIR/$STARTUP_SCRIPT
-do_chgrp $BINGROUP $BINDIR/$STARTUP_SCRIPT
+do_chown $OWNER $BINDIR/$STARTUP_SCRIPT
+do_chgrp $GROUP $BINDIR/$STARTUP_SCRIPT
 do_chmod $SCRPERM $BINDIR/$STARTUP_SCRIPT
 $ECHO "\t$BINDIR/$STARTUP_SCRIPT"
 
 # Use startup wrapper script to invoke all supported programs
-for i in xmcd cda cddbcmd dp2xmcd wm2xmcd
+for i in xmcd cda
 do
 	rm -f $BINDIR/$i
 	do_link $BINDIR/$STARTUP_SCRIPT $BINDIR/$i
@@ -1235,11 +1350,255 @@ then
 		inst_file ${i}_d/${i}.man $MANDIR/${i}.$MANSUFFIX \
 			$FILEPERM $OWNER $GROUP
 	done
-	for i in cddbcmd dp2xmcd wm2xmcd
+fi
+
+# CDDB shared library if bundled
+notfatal="This may not be a fatal problem."
+
+f=`echo cddb_d/libcddb.s*`
+
+if [ -f "$f" ]
+then
+	n=`basename $f`
+	inst_file $f $ARCHLIB/$n $SCRPERM $OWNER $GROUP
+
+	# Look for existing libcddb installation
+	dlist="/usr/lib /lib"
+	if [ -n "$LDSO_CONF" ]
+	then
+		dlist="$dlist `cat $LDSO_CONF 2>/dev/null`"
+	fi
+
+	for d in $dlist
 	do
-		inst_file util_d/${i}.man $MANDIR/${i}.$MANSUFFIX \
-			$FILEPERM $OWNER $GROUP
+		if [ ! -d $d ]
+		then
+			continue
+		fi
+
+		# [ -L file ] is not universally supported hence this
+		# weird syntax in a subshell with stderr directed to /dev/null
+		# Try both sh and ksh
+		if ( test -L $d/$n ) 2>/dev/null || \
+		   ( ksh -c "test -L $d/$n" ) 2>/dev/null
+		then
+			if [ ! -w $d ]
+			then
+				# Existing CDDB shared library symlink
+				# directory not writable
+				$ECHO "\t$d/$n NOT linked"
+				SHLIBEXISTS=$d/$n
+			fi
+			SHLIBDIR=$d
+			break
+		elif [ -f $d/$n ]
+		then
+			if [ ! -w $d/$n ]
+			then
+				# Existing CDDB shared library not writable
+				$ECHO "\t$d/$n NOT linked"
+				SHLIBEXISTS=$d/$n
+			fi
+			SHLIBDIR=$d
+			break
+		fi
 	done
+
+	if [ -z "$SHLIBEXISTS" ]
+	then
+		if [ -z "$SHLIBDIR" ]
+		then
+			if [ "$OS_SYS" = Linux ]
+			then
+				if [ -d /usr/local/lib ]
+				then
+					SHLIBDIR=/usr/local/lib
+				else
+					SHLIBDIR=/usr/lib
+				fi
+			else
+				SHLIBDIR=/usr/lib
+			fi
+		fi
+
+		# Make symlink
+		do_link $ARCHLIB/$n $SHLIBDIR/$n
+		if [ $? -eq 0 ]
+		then
+			$ECHO "\t$SHLIBDIR/$n"
+		else
+			$ECHO "\t$SHLIBDIR/$n NOT linked"
+			if [ $OWNER = _default_ -a $LINKLIBS = False ]
+			then
+				log_err -n "$notfatal"
+			fi
+		fi
+
+		if [ "$OS_SYS" = Linux ]
+		then
+			# Update ld.so.conf if needed
+			fgrep $SHLIBDIR $LDSO_CONF >/dev/null 2>&1
+			if [ $? -ne 0 ]
+			then
+				echo $SHLIBDIR >>$LDSO_CONF 2>/dev/null
+				if [ $? -ne 0 ]
+				then
+					LDSO_NEEDCONF=1
+				fi
+			fi
+
+			# Run ldconfig
+			$LDCONFIG 2>/dev/null
+			if [ $? -ne 0 ]
+			then
+				LDSO_NEEDCONF=1
+			fi
+		elif [ "$OS_SYS" = SunOS ]
+		then
+			case "$OS_REL" in
+			4.*)
+				# SunOS 4.x
+				# Run ldconfig
+				$LDCONFIG 2>/dev/null
+				if [ $? -ne 0 ]
+				then
+					LDSO_NEEDCONF=1
+				fi
+				;;
+			*)
+				;;
+			esac
+		fi
+	fi
+
+	# Make symlink
+	n2="$n"
+	while :
+	do
+		x=`echo "$n2" | sed 's/\.[0-9]*$//'`
+		if [ "$x" = "$n2" ]
+		then
+			break
+		fi
+		n2=$x
+
+		(cd $ARCHLIB; do_link $n $n2) >/dev/null 2>&1
+
+		if [ -z "$SHLIBEXISTS" ]
+		then
+			(cd $SHLIBDIR; do_link $n $n2) >/dev/null 2>&1
+			if [ $? -eq 0 ]
+			then
+				$ECHO "\t$SHLIBDIR/$n2"
+			else
+				$ECHO "\t$SHLIBDIR/$n2 NOT linked"
+				if [ $OWNER = _default_ -a $LINKLIBS = False ]
+				then
+					log_err -n "$notfatal"
+				fi
+			fi
+		fi
+	done
+fi
+
+# Set up local discography directories and files
+if [ -d $DISCOGDIR ]
+then
+	$ECHO "\nSetting up xmcd Local Discography..."
+
+	ows_s=`echo "http://metalab.unc.edu/tkan/xmcd/" | sed 's/\//\\\\\//g'`
+
+	for d in		\
+		blues		\
+		classical	\
+		country		\
+		data		\
+		folk		\
+		jazz		\
+		misc		\
+		newage		\
+		reggae		\
+		rock		\
+		soundtrack
+	do
+	    case $d in
+	    blues)
+		newgenre=Blues/General_Blues
+		;;
+	    classical)
+		newgenre=Classical/General_Classical
+		;;
+	    country)
+		newgenre=Country/General_Country
+		;;
+	    data)
+		newgenre=Data/General_Data
+		;;
+	    folk)
+		newgenre=Folk/General_Folk
+		;;
+	    jazz)
+		newgenre=Jazz/General_Jazz
+		;;
+	    misc)
+		newgenre=Unclassifiable/General_Unclassifiable
+		;;
+	    newage)
+		newgenre=Newage/General_Newage
+		;;
+	    reggae)
+		newgenre=World/Reggae
+		;;
+	    rock)
+		newgenre=Rock/General_Rock
+		;;
+	    soundtrack)
+		newgenre=Soundtrack/General_Soundtrack
+		;;
+	    esac
+
+	    # Make a basic set of xmcd-2.x compatible genre directories
+	    make_dir $DISCOGDIR/$newgenre $GDIRPERM $OWNER $GROUP
+
+	    pdir=`dirname $DISCOGDIR/$newgenre`
+	    do_chown $OWNER $pdir
+	    do_chgrp $GROUP $pdir
+	    do_chmod $GDIRPERM $pdir
+
+	    # Move old xmcd-2.x discography files into new location
+	    if [ -d $DISCOGDIR/$d ]
+	    then
+		for e in $DISCOGDIR/$d/*
+		do
+		    if [ ! -d $e ]
+		    then
+			continue
+		    fi
+
+		    id=`basename $e`
+
+		    if [ `expr "$id" : '[0-9,a-f][0-9,a-f][0-9,a-f][0-9,a-f][0-9,a-f][0-9,a-f][0-9,a-f][0-9,a-f]'` -ne 8 ]
+		    then
+			continue
+		    fi
+
+		    mkdir $DISCOGDIR/$newgenre/$id 2>/dev/null
+		    chmod 777 $DISCOGDIR/$newgenre/$id 2>/dev/null
+
+		    mv $e/* $e/.??* $DISCOGDIR/$newgenre/$id >/dev/null 2>&1
+
+		    # Fix-up up local discography file
+		    fix_discog $d $newgenre $id
+		done
+		rm -rf $DISCOGDIR/$d
+	    fi
+	done
+fi
+
+# Generate local discography index files
+if [ -x $XMCDLIB/scripts/genidx ]
+then
+	$XMCDLIB/scripts/genidx
 fi
 
 if [ -z "$BATCH" ]
@@ -1247,8 +1606,9 @@ then
 	# Run device-dependent config script
 	if [ -r $XMCDLIB/config/config.sh ]
 	then
+		FROM_INSTALL_SH=1; export FROM_INSTALL_SH
 		$SHELL $XMCDLIB/config/config.sh
-		if [ $? != 0 ]
+		if [ $? -ne 0 ]
 		then
 			log_err -n "$XMCDLIB/config/config.sh failed."
 		fi
@@ -1257,14 +1617,6 @@ then
 	fi
 fi
 
-# Demo cddb file
-if [ -d $CDDBDIR/misc ]
-then
-	$ECHO "\nInstalling demo mode CDDB file..."
-	inst_file misc_d/demo.db $CDDBDIR/misc/$DEMODB \
-		$FILEPERM $OWNER $GROUP
-fi
-
 # Set up dynamic library links
 if [ $LINKLIBS = True ]
 then
@@ -1274,8 +1626,9 @@ fi
 # Clean up
 if [ $NOREMOVE = 0 ]
 then
-	rm -rf common_d libdi_d misc_d xmcd_d cda_d util_d docs_d install.sh \
-		xmcd.tar
+	rm -rf  common_d libdi_d misc_d xmcd_d cda_d \
+		cdinfo_d docs_d cddb_d util_d install.sh \
+		xmcdbin.tar xmcd.tar
 fi
 
 do_exit $ERROR
