@@ -1,22 +1,22 @@
 /*
  *   libdi - scsipt SCSI Device Interface Library
  *
- *   Copyright (C) 1993-2000  Ti Kan
- *   E-mail: ti@amb.org
+ *   Copyright (C) 1993-2001  Ti Kan
+ *   E-mail: xmcd@amb.org
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Library General Public
- *   License as published by the Free Software Foundation; either
- *   version 2 of the License, or (at your option) any later version.
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
  *
- *   This library is distributed in the hope that it will be useful,
+ *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *   Library General Public License for more details.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
  *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this library; if not, write to the Free
- *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program; if not, write to the Free Software
+ *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
 /*
@@ -29,7 +29,7 @@
  *   is used here for identification purposes only.
  */
 #ifndef LINT
-static char *_os_aux_c_ident_ = "@(#)os_aux.c	6.24 99/12/27";
+static char *_os_aux_c_ident_ = "@(#)os_aux.c	6.29 01/04/17";
 #endif
 
 #include "common_d/appenv.h"
@@ -109,7 +109,7 @@ pthru_send(
 	}
 	sc_cmd.timeout = tmout ? tmout : DFLT_CMD_TIMEOUT;
 
-	if (app_data.debug) {
+	if (app_data.debug & DBG_DEVIO) {
 		(void) sprintf(title, "SCSI CDB bytes (dev=%s rw=%d to=%d)",
 			       path, rw, tmout);
 		util_dbgdump(title, cmdpt, cmdlen);
@@ -197,7 +197,8 @@ pthru_open(char *path)
 	}
 
 	if ((fd = open(path, O_RDONLY, NULL)) < 0) {
-		DBGPRN(errfp, "Cannot open %s: errno=%d\n", path, errno);
+		DBGPRN(DBG_DEVIO)(errfp,
+			"Cannot open %s: errno=%d\n", path, errno);
 		return -1;
 	}
 
@@ -237,7 +238,7 @@ pthru_close(int fd)
 char *
 pthru_vers(void)
 {
-	return ("OS Interface module for Apple A/UX 3.x\n");
+	return ("OS Interface module for Apple A/UX 3.x");
 }
 
 #endif	/* macII DI_SCSIPT DEMO_ONLY */
