@@ -1,25 +1,25 @@
 /*
  *   libdi - scsipt SCSI Device Interface Library
  *
- *   Copyright (C) 1993-2000  Ti Kan
- *   E-mail: ti@amb.org
+ *   Copyright (C) 1993-2001  Ti Kan
+ *   E-mail: xmcd@amb.org
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Library General Public
- *   License as published by the Free Software Foundation; either
- *   version 2 of the License, or (at your option) any later version.
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
  *
- *   This library is distributed in the hope that it will be useful,
+ *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *   Library General Public License for more details.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
  *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this library; if not, write to the Free
- *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program; if not, write to the Free Software
+ *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #ifndef LINT
-static char *_os_cdsim_c_ident_ = "@(#)os_cdsim.c	6.19 99/12/27";
+static char *_os_cdsim_c_ident_ = "@(#)os_cdsim.c	6.23 01/04/09";
 #endif
 
 #include "common_d/appenv.h"
@@ -676,10 +676,11 @@ cdsim_svccmd(simpkt_t *r, simpkt_t *s)
 {
 	(void) memset(s, 0, CDSIM_PKTSZ);
 
-	DBGPRN(errfp, "\ncdsim: pktid=%d cdbsz=%d len=%d dir=%d",
+	DBGPRN(DBG_DEVIO)(errfp, "\ncdsim: pktid=%d cdbsz=%d len=%d dir=%d",
 		r->pktid, r->cdbsz, r->len, r->dir);
 
-	DBGDUMP("cdsim: SCSI CDB bytes", (byte_t *) r->cdb, (int) r->cdbsz);
+	DBGDUMP(DBG_DEVIO)("cdsim: SCSI CDB bytes",
+		(byte_t *) r->cdb, (int) r->cdbsz);
 
 	/* Set return packet id */
 	s->pktid = r->pktid;
