@@ -1,22 +1,22 @@
 /*
  *   libdi - CD Audio Player Device Interface Library
  *
- *   Copyright (C) 1993-2000  Ti Kan
- *   E-mail: ti@amb.org
+ *   Copyright (C) 1993-2001  Ti Kan
+ *   E-mail: xmcd@amb.org
  *
- *   This library is free software; you can redistribute it and/or
- *   modify it under the terms of the GNU Library General Public
- *   License as published by the Free Software Foundation; either
- *   version 2 of the License, or (at your option) any later version.
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
  *
- *   This library is distributed in the hope that it will be useful,
+ *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
- *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- *   Library General Public License for more details.
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
  *
- *   You should have received a copy of the GNU Library General Public
- *   License along with this library; if not, write to the Free
- *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program; if not, write to the Free Software
+ *   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 #ifndef __SCSIPT_H__
 #define __SCSIPT_H__
@@ -24,7 +24,7 @@
 #ifdef DI_SCSIPT
 
 #ifndef LINT
-static char *_scsipt_h_ident_ = "@(#)scsipt.h	6.35 00/01/18";
+static char *_scsipt_h_ident_ = "@(#)scsipt.h	6.39 01/04/17";
 #endif
 
 
@@ -897,8 +897,7 @@ extern void	scsipt_mute_off(curstat_t *);
 extern void	scsipt_start(curstat_t *);
 extern void	scsipt_icon(curstat_t *, bool_t);
 extern void	scsipt_halt(curstat_t *);
-extern char	*scsipt_mode(void);
-extern char	*scsipt_vers(void);
+extern char	*scsipt_methodstr(void);
 
 /*
  * Functions for vendor-unique module use only
