@@ -1,14 +1,14 @@
 #!/bin/sh
 #
-# @(#)makerel.sh	6.13 99/12/27
+# @(#)makerel.sh	6.23 01/04/17
 #
 # Make software binary release
 #
 #    xmcd  - Motif(tm) CD Audio Player
 #    cda   - Command-line CD Audio Player
 #
-#    Copyright (C) 1993-2000  Ti Kan
-#    E-mail: ti@amb.org
+#    Copyright (C) 1993-2001  Ti Kan
+#    E-mail: xmcd@amb.org
 #
 #    This program is free software; you can redistribute it and/or modify
 #    it under the terms of the GNU General Public License as published by
@@ -28,6 +28,7 @@
 # The file containing a list of binary release files
 BINLIST=misc_d/BINLIST
 TMPDIR=/tmp/_makerel_d
+TOPDIR=xmcdbin
 ZFILE=xmcdbin.tar.gz
 UUEFILE=xmcdbin.uue
 COMPRESS=gzip
@@ -83,28 +84,66 @@ mkdir -p $TMPDIR
 for i in `awk '!/^#/ { print $1 }' $BINLIST`
 do
 	$ECHO "\t$i"
-	DEST=`dirname $TMPDIR/$i`
+	DEST=`dirname $TMPDIR/$TOPDIR/$i`
 	mkdir -p $DEST >/dev/null 2>&1
 	cp $i $DEST
 done
 
+# Copy CDDB2 shared library into binary bundle
+case `uname -s` in
+HP-UX)
+	shlib=libcddb.sl
+	;;
+SunOS)
+	case `uname -r` in
+	4*)
+		shlib=libcddb.so.1.0
+		;;
+	5*)
+		shlib=libcddb.so.1
+		;;
+	*)
+		shlib=libcddb.so.1
+		;;
+	esac
+	;;
+*)
+	shlib=libcddb.so.1
+	;;
+esac
+
+for d in /usr/lib /usr/local/lib /lib /usr/ccs/lib
+do
+	if [ -f $d/$shlib ]
+	then
+		DEST=$TMPDIR/$TOPDIR/cddb_d
+		mkdir -p $DEST >/dev/null 2>&1
+		cp $d/$shlib $DEST
+		$ECHO "\tcddb_d/$shlib"
+		break
+	fi
+done
+
+if [ ! -d $TMPDIR/$TOPDIR/cddb_d ]
+then
+	$ECHO "\nNOTICE: CDDB shared library not included."
+fi
+
 # Strip the binary symbol tables
-strip $TMPDIR/xmcd_d/xmcd >/dev/null 2>&1
-strip $TMPDIR/cda_d/cda >/dev/null 2>&1
-strip $TMPDIR/util_d/wm2xmcd >/dev/null 2>&1
-strip $TMPDIR/util_d/cddbcmd >/dev/null 2>&1
+strip $TMPDIR/$TOPDIR/xmcd_d/xmcd >/dev/null 2>&1
+strip $TMPDIR/$TOPDIR/cda_d/cda >/dev/null 2>&1
+strip $TMPDIR/$TOPDIR/util_d/gobrowser >/dev/null 2>&1
 
 # Remove comment section of binaries if possible
-(mcs -da "@(#)xmcd $VERS (C) Ti Kan 1993-2000" \
-	$TMPDIR/xmcd_d/xmcd) >/dev/null 2>&1
-(mcs -da "@(#)cda $VERS (C) Ti Kan 1993-2000" \
-	$TMPDIR/cda_d/cda) >/dev/null 2>&1
-(mcs -da "@(#)wm2xmcd $VERS (C) Ti Kan 1993-2000" \
-	$TMPDIR/util_d/wm2xmcd) >/dev/null 2>&1
-(mcs -da "@(#)cddbcmd $VERS (C) Ti Kan 1993-2000" \
-	$TMPDIR/util_d/cddbcmd) >/dev/null 2>&1
+(mcs -da "@(#)xmcd $VERS (C) Ti Kan 1993-2001" \
+	$TMPDIR/$TOPDIR/xmcd_d/xmcd) >/dev/null 2>&1
+(mcs -da "@(#)cda $VERS (C) Ti Kan 1993-2001" \
+	$TMPDIR/$TOPDIR/cda_d/cda) >/dev/null 2>&1
+(mcs -da "@(#)gobrowser $VERS" \
+	$TMPDIR/$TOPDIR/util_d/gobrowser) >/dev/null 2>&1
 
 $ECHO "\nFixing permissions..."
+(cd $TMPDIR; find * -type d -print | xargs chmod 755)
 (cd $TMPDIR; find * -type f -print | xargs chmod 444)
 
 $ECHO "\nCreating \"$COMPRESS\"ed tar archive..."
@@ -118,13 +157,13 @@ Instructions to unpack xmcd v_VERS_ binary and other info
 ------------------------------------------------------
 
 At the end of this message is a "_COMPRESS_"ed and "_ENCODE_"ed
-tar file containing the xmcd, cda, wm2xmcd and cddbcmd binaries
-as well as their supporting files.  To extract, save this
-message in a file "_UUEFILE_" and do the following while
-logged in as root:
+tar file containing the xmcd and cda executables as well as
+their supporting files.  To extract, save this message in a
+file "_UUEFILE_", login as root and do the following:
 
 	_DECODE_ _UUEFILE_
 	_UNCOMPRESS_ < _ZFILE_ | tar xvf -
+	cd _TOPDIR_
 	sh ./install.sh
 
 Be sure to run the "install.sh" script to install and configure
@@ -136,6 +175,7 @@ installation.
 ' | sed	-e "s/_VERS_/$VERS/g" \
 	-e "s/_UUEFILE_/$UUEFILE/g" \
 	-e "s/_ZFILE_/$ZFILE/g" \
+	-e "s/_TOPDIR_/$TOPDIR/g" \
 	-e "s/_COMPRESS_/$COMPRESS/g" \
 	-e "s/_UNCOMPRESS_/$UNCOMPRESS/g" \
 	-e "s/_ENCODE_/$ENCODE/g" \
