@@ -1,8 +1,8 @@
 /*
  *   xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-2000  Ti Kan
- *   E-mail: ti@amb.org
+ *   Copyright (C) 1993-2001  Ti Kan
+ *   E-mail: xmcd@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,14 +20,14 @@
  *
  */
 #ifndef LINT
-static char *_command_c_ident_ = "@(#)command.c	6.32 00/01/23";
+static char *_command_c_ident_ = "@(#)command.c	7.16 01/04/09";
 #endif
 
 #include "common_d/appenv.h"
 #include "common_d/patchlevel.h"
 #include "common_d/util.h"
 #include "libdi_d/libdi.h"
-#include "cddb_d/cddb.h"
+#include "cdinfo_d/cdinfo.h"
 #include "xmcd_d/xmcd.h"
 #include "xmcd_d/widget.h"
 #include "xmcd_d/cdfunc.h"
@@ -44,7 +44,7 @@ extern FILE		*errfp;
 
 /* Log file banner */
 #define RMTLOG_BANNER	\
-"# xmcd %s Remote control log file\n# %s\n#\n\
+"# xmcd %s.%s Remote control log file\n# %s\n#\n\
 # YY/MM/DD HH:MM:SS host/dev/pid/display user@host/pid command\n#\n"
 
 #ifdef __VMS
@@ -212,8 +212,6 @@ cmd_do_play(Display *dpy, curstat_t *s, char *arg, bool_t prn_usage)
 		switch (s->mode) {
 		case MOD_PAUSE:
 		case MOD_PLAY:
-		case MOD_A:
-		case MOD_AB:
 		case MOD_SAMPLE:
 			sav_cur_trk = s->cur_trk;
 
@@ -230,16 +228,16 @@ cmd_do_play(Display *dpy, curstat_t *s, char *arg, bool_t prn_usage)
 			break;
 		}
 
-		s->cur_trk_min = (byte_t) min;
-		s->cur_trk_sec = (byte_t) sec;
-		s->cur_trk_frame = 0;
-		s->cur_trk_addr = addr;
-		s->cur_tot_addr = s->trkinfo[i].addr + s->cur_trk_addr;
+		s->curpos_trk.min = (byte_t) min;
+		s->curpos_trk.sec = (byte_t) sec;
+		s->curpos_trk.frame = 0;
+		s->curpos_trk.addr = addr;
+		s->curpos_tot.addr = s->trkinfo[i].addr + s->curpos_trk.addr;
 		util_blktomsf(
-			s->cur_tot_addr,
-			&s->cur_tot_min,
-			&s->cur_tot_sec,
-			&s->cur_tot_frame,
+			s->curpos_tot.addr,
+			&s->curpos_tot.min,
+			&s->curpos_tot.sec,
+			&s->curpos_tot.frame,
 			MSF_OFFSET
 		);
 	}
@@ -471,8 +469,6 @@ cmd_do_track(Display *dpy, curstat_t *s, char *arg, bool_t prn_usage)
 
 		/*FALLTHROUGH*/
 	case MOD_PLAY:
-	case MOD_A:
-	case MOD_AB:
 	case MOD_SAMPLE:
 		playing = TRUE;
 		sav_cur_trk = s->cur_trk;
@@ -491,20 +487,23 @@ cmd_do_track(Display *dpy, curstat_t *s, char *arg, bool_t prn_usage)
 		break;
 	}
 
+	s->segplay = SEGP_NONE;	/* Cancel a->b mode */
+	dpy_progmode(s, FALSE);
+
 	if (playing) {
-		s->cur_trk_min = 0;
-		s->cur_trk_sec = 0;
-		s->cur_trk_frame = 0;
-		s->cur_trk_addr = 0;
-		s->cur_tot_min = s->trkinfo[i].min;
-		s->cur_tot_sec = s->trkinfo[i].sec;
-		s->cur_tot_frame = s->trkinfo[i].frame;;
-		s->cur_tot_addr = 0;
+		s->curpos_trk.min = 0;
+		s->curpos_trk.sec = 0;
+		s->curpos_trk.frame = 0;
+		s->curpos_trk.addr = 0;
+		s->curpos_tot.min = s->trkinfo[i].min;
+		s->curpos_tot.sec = s->trkinfo[i].sec;
+		s->curpos_tot.frame = s->trkinfo[i].frame;;
+		s->curpos_tot.addr = 0;
 		util_msftoblk(
-			s->cur_tot_min,
-			s->cur_tot_sec,
-			s->cur_tot_frame,
-			&s->cur_tot_addr,
+			s->curpos_tot.min,
+			s->curpos_tot.sec,
+			s->curpos_tot.frame,
+			&s->curpos_tot.addr,
 			MSF_OFFSET
 		);
 
@@ -718,12 +717,12 @@ cmd_do_repeat(Display *dpy, curstat_t *s, char *arg, bool_t prn_usage)
 STATIC int
 cmd_do_program(Display *dpy, curstat_t *s, char *arg, bool_t prn_usage)
 {
-	char	*p;
-
 	if (prn_usage) {
-		(void) fprintf(errfp, "\tprogram <clear | track# ...>\n");
+		(void) fprintf(errfp,
+			"\tprogram <clear | save | track# ...>\n");
 		return CMD_STAT_SUCCESS;
 	}
+
 	if (strcmp(arg, "clear") == 0) {
 		if (s->program) {
 			XtCallCallbacks(
@@ -734,6 +733,16 @@ cmd_do_program(Display *dpy, curstat_t *s, char *arg, bool_t prn_usage)
 		return CMD_STAT_SUCCESS;
 	}
 
+	if (strcmp(arg, "save") == 0) {
+		if (s->program) {
+			XtCallCallbacks(
+				widgets.dbprog.savepgm_btn,
+				XmNactivateCallback, (XtPointer) s
+			);
+		}
+		return CMD_STAT_SUCCESS;
+	}
+
 	if (s->mode != MOD_NODISC && s->mode != MOD_BUSY) {
 		if (s->program) {
 			/* Clear program first */
@@ -743,12 +752,6 @@ cmd_do_program(Display *dpy, curstat_t *s, char *arg, bool_t prn_usage)
 			);
 		}
 
-		/* Change spaces into commas */
-		for (p = arg; *p != '\0'; p++) {
-			if (*p == ' ')
-				*p = ',';
-		}
-
 		/* Set specified program */
 		XmTextSetString(widgets.dbprog.pgmseq_txt, arg);
 
@@ -810,14 +813,14 @@ cmd_do_volume(Display *dpy, curstat_t *s, char *arg, bool_t prn_usage)
 		return CMD_STAT_SUCCESS;
 	}
 
-	if (!isdigit(arg[0]))
+	if (!isdigit((int) arg[0]))
 		return CMD_STAT_BADARG;
 
 	val = atoi(arg);
 	if (val < 0 || val > 100)
 		return CMD_STAT_BADARG;
 
-	DBGPRN(errfp, "\n* VOL\n");
+	DBGPRN(DBG_UI)(errfp, "\n* VOL\n");
 
 	set_vol_slider(val);
 	di_level(s, (byte_t) val, FALSE);
@@ -851,7 +854,7 @@ cmd_do_balance(Display *dpy, curstat_t *s, char *arg, bool_t prn_usage)
 		return CMD_STAT_SUCCESS;
 	}
 
-	if (!isdigit(arg[0]))
+	if (!isdigit((int) arg[0]))
 		return CMD_STAT_BADARG;
 
 	val = atoi(arg);
@@ -964,16 +967,20 @@ STATIC int
 cmd_do_time(Display *dpy, curstat_t *s, char *arg, bool_t prn_usage)
 {
 	if (prn_usage) {
-		(void) fprintf(errfp,
-			"\ttime <elapse | e-disc | r-trac | r-disc>\n");
+		(void) fprintf(errfp, "\ttime %s\n",
+		    "<elapse | e-seg | e-disc | r-trac | r-seg | r-disc>");
 		return CMD_STAT_SUCCESS;
 	}
 	if (strcmp(arg, "elapse") == 0)
-		s->time_dpy = T_ELAPSED;
+		s->time_dpy = T_ELAPSED_TRACK;
+	else if (strcmp(arg, "e-seg") == 0)
+		s->time_dpy = T_ELAPSED_SEG;
 	else if (strcmp(arg, "e-disc") == 0)
 		s->time_dpy = T_ELAPSED_DISC;
 	else if (strcmp(arg, "r-track") == 0)
 		s->time_dpy = T_REMAIN_TRACK;
+	else if (strcmp(arg, "r-seg") == 0)
+		s->time_dpy = T_REMAIN_SEG;
 	else if (strcmp(arg, "r-disc") == 0)
 		s->time_dpy = T_REMAIN_DISC;
 	else
@@ -1356,24 +1363,21 @@ STATIC int
 cmd_do_debug(Display *dpy, curstat_t *s, char *arg, bool_t prn_usage)
 {
 	if (prn_usage) {
-		(void) fprintf(errfp, "\tdebug <on | off>\n");
+		(void) fprintf(errfp, "\tdebug <level#>\n");
 		return CMD_STAT_SUCCESS;
 	}
-	if (strcmp(arg, "on") == 0) {
-		if (!app_data.debug) {
-			app_data.debug = TRUE;
-			(void) fprintf(errfp, "Enabling DEBUG mode.\n");
-		}
-		return CMD_STAT_SUCCESS;
-	}
-	if (strcmp(arg, "off") == 0) {
-		if (app_data.debug) {
-			app_data.debug = FALSE;
-			(void) fprintf(errfp, "Disabling DEBUG mode.\n");
-		}
-		return CMD_STAT_SUCCESS;
-	}
-	return CMD_STAT_BADARG;
+
+	if (!isdigit((int) arg[0]))
+		return CMD_STAT_BADARG;
+
+	app_data.debug = (word32_t) atoi(arg);
+	if (app_data.debug != 0)
+		(void) fprintf(errfp, "Enabling DEBUG: level=0x%x.\n",
+				app_data.debug);
+	else
+		(void) fprintf(errfp, "Disabling DEBUG.\n");
+
+	return CMD_STAT_SUCCESS;
 }
 
 
@@ -1615,7 +1619,8 @@ cmd_getwin(Display *dpy)
 		    type != XA_STRING || idstr == NULL)
 			continue;
 
-		DBGPRN(errfp, "Found \"%s\" window 0x%x\n", idstr, (int) w);
+		DBGPRN(DBG_RMT)(errfp,
+			"Found \"%s\" window 0x%x\n", idstr, (int) w);
 
 		/* Locate the host and device */
 		for (p = idstr, j = 0; j < 3; j++, p++) {
@@ -1642,7 +1647,8 @@ cmd_getwin(Display *dpy)
 		if (app_data.remotehost != NULL &&
 		    app_data.remotehost[0] != '\0') {
 
-			DBGPRN(errfp, "Comparing host \"%s\" : \"%s\"\n",
+			DBGPRN(DBG_RMT)(errfp,
+				"Comparing host \"%s\" : \"%s\"\n",
 				host, app_data.remotehost);
 
 			if (HOST_STRCMP(host, app_data.remotehost) != 0) {
@@ -1660,7 +1666,8 @@ cmd_getwin(Display *dpy)
 			else
 				(void) strcpy(dev, device);
 
-			DBGPRN(errfp, "Comparing device \"%s\" : \"%s\"\n",
+			DBGPRN(DBG_RMT)(errfp,
+				"Comparing device \"%s\" : \"%s\"\n",
 				dev, app_data.device);
 
 			if (DEV_STRCMP(dev, app_data.device) != 0) {
@@ -1676,7 +1683,8 @@ cmd_getwin(Display *dpy)
 				(void) sprintf(dev, "%s$%s",
 						host, app_data.device);
 
-			DBGPRN(errfp, "Comparing device \"%s\" : \"%s\"\n",
+			DBGPRN(DBG_RMT)(errfp,
+				"Comparing device \"%s\" : \"%s\"\n",
 				device, dev);
 
 			if (DEV_STRCMP(device, dev) != 0) {
@@ -1686,7 +1694,7 @@ cmd_getwin(Display *dpy)
 			}
 		}
 #else
-		DBGPRN(errfp, "Comparing device \"%s\" : \"%s\"\n",
+		DBGPRN(DBG_RMT)(errfp, "Comparing device \"%s\" : \"%s\"\n",
 			device, app_data.device);
 
 		if (DEV_STRCMP(device, app_data.device) != 0) {
@@ -1699,7 +1707,7 @@ cmd_getwin(Display *dpy)
 		XFree(idstr);
 
 		/* Found client */
-		DBGPRN(errfp, "Using window 0x%x\n", (int) w);
+		DBGPRN(DBG_RMT)(errfp, "Using window 0x%x\n", (int) w);
 		return (w);
  	}
 
@@ -1757,7 +1765,7 @@ cmd_attach(Display *dpy, Window win)
 			&type, &fmt, &n, &resid,
 			(unsigned char **) &attstr) != Success ||
 		    type != XA_STRING || attstr == NULL) {
-			DBGPRN(errfp, "Attaching to window 0x%x\n",
+			DBGPRN(DBG_RMT)(errfp, "Attaching to window 0x%x\n",
 				(int) win);
 
 			XChangeProperty(
@@ -1778,7 +1786,8 @@ cmd_attach(Display *dpy, Window win)
 		if (attstr != NULL)
 			XFree(attstr);
 
-		DBGPRN(errfp, "Window 0x%x is attached by %s; Waiting...\n",
+		DBGPRN(DBG_RMT)(errfp,
+			"Window 0x%x is attached by %s; Waiting...\n",
 			(int) win, attstr);
 
 		/* Wait until detach or destroy */
@@ -1786,12 +1795,14 @@ cmd_attach(Display *dpy, Window win)
 			XNextEvent(dpy, &ev);
 
 			if (EV_PROP_NOTIFY(&ev, win, PropertyDelete, cmd_att)) {
-				DBGPRN(errfp, "Window 0x%x detached.\n",
+				DBGPRN(DBG_RMT)(errfp,
+					"Window 0x%x detached.\n",
 					(int) win);
 				break;
 			}
 			else if (EV_DESTROY_WIN(&ev, win)) {
-				DBGPRN(errfp, "Window 0x%x was destroyed.\n",
+				DBGPRN(DBG_RMT)(errfp,
+					"Window 0x%x was destroyed.\n",
 					(int) win);
 				return 1;
 			}
@@ -1826,11 +1837,12 @@ cmd_detach(Display *dpy, Window win)
 		True, XA_STRING,  &type, &fmt, &n, &resid,
 		(unsigned char **) &attstr) != Success ||
 	    type != XA_STRING || attstr == NULL) {
-		DBGPRN(errfp, "Cannot detach from window 0x%x.\n", (int) win);
+		DBGPRN(DBG_RMT)(errfp,
+			"Cannot detach from window 0x%x.\n", (int) win);
 		return;
 	}
 
-	if (app_data.debug) {
+	if (app_data.debug & DBG_RMT) {
 		if (attstr == NULL || *attstr == '\0') {
 			(void) fprintf(errfp,
 				"Null att property data on window 0x%x.\n",
@@ -1848,7 +1860,7 @@ cmd_detach(Display *dpy, Window win)
 	if (attstr != NULL)
 		XFree(attstr);
 
-	DBGPRN(errfp, "Detached from window 0x%x\n", (int) win);
+	DBGPRN(DBG_RMT)(errfp, "Detached from window 0x%x\n", (int) win);
 }
 
 
@@ -1867,7 +1879,8 @@ cmd_detach(Display *dpy, Window win)
 STATIC void
 cmd_putreq(Display *dpy, Window win, char *str)
 {
-	DBGPRN(errfp, "Sending request (%s): %s\n", DisplayString(dpy), str);
+	DBGPRN(DBG_RMT)(errfp,
+		"Sending request (%s): %s\n", DisplayString(dpy), str);
 
 	XChangeProperty(
 		dpy, win,
@@ -1915,7 +1928,7 @@ cmd_getack(Display *dpy, Window win, char *str)
 			    fmt != 32 ||
 			    type != XA_INTEGER ||
 			    ackp == NULL) {
-				DBGPRN(errfp,
+				DBGPRN(DBG_RMT)(errfp,
 					"Cannot get ack from window 0x%0x.\n",
 					(int) win
 				);
@@ -1925,15 +1938,16 @@ cmd_getack(Display *dpy, Window win, char *str)
 			ret = *ackp;
 			XFree((char *) ackp);
 
-			DBGPRN(errfp, "Remote request status code %d: ", ret);
+			DBGPRN(DBG_RMT)(errfp,
+				"Remote request status code %d: ", ret);
 
 			switch (ret) {
 			case CMD_STAT_SUCCESS:
-				DBGPRN(errfp, "Completed\n");
+				DBGPRN(DBG_RMT)(errfp, "Completed\n");
 				return (ret);
 
 			case CMD_STAT_PROCESSING:
-				DBGPRN(errfp, "Processing\n");
+				DBGPRN(DBG_RMT)(errfp, "Processing\n");
 				break;
 
 			case CMD_STAT_UNSUPP:
@@ -1959,14 +1973,14 @@ cmd_getack(Display *dpy, Window win, char *str)
 				return (ret);
 
 			default:
-				DBGPRN(errfp,
+				DBGPRN(DBG_RMT)(errfp,
 				    "Unrecognized ack from window 0x%x: %d\n",
 				    (int) win, ret);
 				break;
 			}
 		}
 		else if (EV_DESTROY_WIN(&ev, win)) {
-			DBGPRN(errfp,
+			DBGPRN(DBG_RMT)(errfp,
 				"%s: window 0x%x was destroyed.\n",
 				"Failed to send remote request",
 				(int) win
@@ -2009,7 +2023,7 @@ cmd_rmtlog(Display *dpy, char *attstr, char *reqstr)
 	waitret_t	stat_val;
 #endif
 
-	DBGPRN(errfp, "%s\n", reqstr);
+	DBGPRN(DBG_RMT)(errfp, "%s\n", reqstr);
 
 	if (!app_data.remote_log)
 		return;
@@ -2028,7 +2042,8 @@ cmd_rmtlog(Display *dpy, char *attstr, char *reqstr)
 		break;
 
 	case -1:
-		DBGPRN(errfp, "cmd_rmtlog: fork failed (errno=%d)\n", errno);
+		DBGPRN(DBG_RMT)(errfp,
+			"cmd_rmtlog: fork failed (errno=%d)\n", errno);
 		return;
 
 	default:
@@ -2041,7 +2056,7 @@ cmd_rmtlog(Display *dpy, char *attstr, char *reqstr)
 			event_loop(0);
 		}
 
-		if (app_data.debug) {
+		if (app_data.debug & DBG_RMT) {
 			if (WIFEXITED(stat_val)) {
 				(void) fprintf(errfp,
 					"cmd_rmtlog: child exit status %d\n",
@@ -2068,7 +2083,7 @@ cmd_rmtlog(Display *dpy, char *attstr, char *reqstr)
 
 	(void) sprintf(path, USR_RMTLOG_PATH, homep);
 
-	DBGPRN(errfp, "Writing remote log file %s\n", path);
+	DBGPRN(DBG_GEN)(errfp, "Writing remote log file %s\n", path);
 
 	/* Check the path */
 	newfile = FALSE;
@@ -2076,14 +2091,14 @@ cmd_rmtlog(Display *dpy, char *attstr, char *reqstr)
 		if (errno == ENOENT)
 			newfile = TRUE;
 		else {
-			DBGPRN(errfp, "Cannot stat %s (errno=%d)\n",
+			DBGPRN(DBG_GEN)(errfp, "Cannot stat %s (errno=%d)\n",
 				path, errno);
 			RL_RET(1);
 		}
 	}
 	else {
 		if (!S_ISREG(stbuf.st_mode)) {
-			DBGPRN(errfp,
+			DBGPRN(DBG_GEN)(errfp,
 				"Not a regular file error: %s\n", path);
 			RL_RET(1);
 		}
@@ -2094,7 +2109,7 @@ cmd_rmtlog(Display *dpy, char *attstr, char *reqstr)
 
 	/* Get a log file handle */
 	if ((fp = fopen(path, "a")) == NULL) {
-		DBGPRN(errfp, "Cannot open file: %s\n", path);
+		DBGPRN(DBG_GEN)(errfp, "Cannot open file: %s\n", path);
 		RL_RET(1);
 	}
 
@@ -2103,7 +2118,8 @@ cmd_rmtlog(Display *dpy, char *attstr, char *reqstr)
 
 	if (newfile)
 		/* Write remote log file header */
-		(void) fprintf(fp, RMTLOG_BANNER, VERSION, COPYRIGHT);
+		(void) fprintf(fp, RMTLOG_BANNER,
+			       VERSION_MAJ, VERSION_MIN, COPYRIGHT);
 
 	(void) time(&t);
 	tm = localtime(&t);
@@ -2138,14 +2154,14 @@ cmd_rmtlog(Display *dpy, char *attstr, char *reqstr)
 		(void) fclose(fp);
 		errno = ret;
 
-		DBGPRN(errfp, "File write error: %s (errno=%d)\n",
+		DBGPRN(DBG_GEN)(errfp, "File write error: %s (errno=%d)\n",
 			path, errno);
 		RL_RET(1);
 	}
 
 	/* Close file */
 	if (fclose(fp) != 0) {
-		DBGPRN(errfp, "File close error: %s (errno=%d)\n",
+		DBGPRN(DBG_GEN)(errfp, "File close error: %s (errno=%d)\n",
 			path, errno);
 		RL_RET(1);
 	}
@@ -2175,21 +2191,61 @@ void
 cmd_init(curstat_t *s, Display *dpy, bool_t sender)
 {
 	struct utsname	*up;
-	char		str[MAX_IDENT_LEN + 1];
+	char		*cp,
+			*hd,
+			str[MAX_IDENT_LEN + 1];
 
 	if (sender) {
-		DBGPRN(errfp, "XMCD v%s%s PL%d DEBUG MODE (remote sender)\n",
-		       VERSION, VERSION_EXT, PATCHLEVEL);
+		DBGPRN(DBG_ALL)(errfp,
+			"XMCD %s.%s%s PL%d DEBUG MODE (remote sender)\n",
+			VERSION_MAJ, VERSION_MIN, VERSION_EXT, PATCHLEVEL);
+
+		if ((cp = (char *) getenv("XMCD_LIBDIR")) == NULL) {
+			/* No library directory specified */
+			if (di_isdemo()) {
+				/* Demo mode: just fake it */
+				app_data.libdir = ".";
+			}
+			else {
+				/* Real application: this is a fatal error */
+				CD_FATAL(app_data.str_libdirerr);
+				return;
+			}
+		}
+		else if (!util_newstr(&app_data.libdir, cp)) {
+			CD_FATAL(app_data.str_nomemory);
+			return;
+		}
+
+		/* Paranoia: avoid overflowing buffers */
+		if ((int) strlen(app_data.libdir) >= FILE_PATH_SZ) {
+			CD_FATAL(app_data.str_longpatherr);
+			return;
+		}
+		hd = util_homedir(util_get_ouid());
+		if ((int) strlen(hd) >= FILE_PATH_SZ) {
+			CD_FATAL(app_data.str_longpatherr);
+			return;
+		}
 
 		/* Get system common configuration parameters */
 		(void) sprintf(str, SYS_CMCFG_PATH, app_data.libdir);
 		di_common_parmload(str, TRUE);
 
 		/* Get user common configuration parameters */
-		(void) sprintf(str, USR_CMCFG_PATH,
-				util_homedir(util_get_ouid()));
+		(void) sprintf(str, USR_CMCFG_PATH, hd);
 		di_common_parmload(str, FALSE);
 
+		/* Paranoia: avoid overflowing buffers */
+		if (app_data.device != NULL) {
+			if ((int) strlen(app_data.device) >= FILE_PATH_SZ ||
+			    (int) strlen(util_basename(app_data.device)) >=
+			    FILE_BASE_SZ) {
+				CD_FATAL(app_data.str_longpatherr);
+				return;
+			}
+		}
+
 		if (!app_data.remote_enb)
 			return;
 	}
@@ -2213,8 +2269,9 @@ cmd_init(curstat_t *s, Display *dpy, bool_t sender)
 			up = util_get_uname();
 
 			/* Set identification property */
-			(void) sprintf(str, "%s %s PL%d %s %s",
-					PROGNAME, VERSION, PATCHLEVEL,
+			(void) sprintf(str, "%s %s.%s PL%d %s %s",
+					PROGNAME, VERSION_MAJ, VERSION_MIN,
+					PATCHLEVEL,
 					up->nodename,
 					app_data.device);
 
@@ -2246,15 +2303,16 @@ cmd_startup(byte_t *cmdstr)
 	curstat_t	*s = curstat_addr();
 	int		ret;
 
-	DBGPRN(errfp, "\nExecuting startup command: %s\n", (char *) cmdstr);
+	DBGPRN(DBG_RMT)(errfp,
+		"\nExecuting startup command: %s\n", (char *) cmdstr);
 
 	ret = cmd_parse_str(dpy, (char *) cmdstr, s);
 
-	DBGPRN(errfp, "Command status %d: ", ret);
+	DBGPRN(DBG_RMT)(errfp, "Command status %d: ", ret);
 
 	switch (ret) {
 	case CMD_STAT_SUCCESS:
-		DBGPRN(errfp, "Success\n");
+		DBGPRN(DBG_RMT)(errfp, "Success\n");
 		break;
 
 	case CMD_STAT_UNSUPP:
@@ -2275,7 +2333,7 @@ cmd_startup(byte_t *cmdstr)
 
 	default:
 		/* These should not happen in this mode */
-		DBGPRN(errfp, "???\n");
+		DBGPRN(DBG_RMT)(errfp, "???\n");
 		break;
 	}
 
@@ -2310,7 +2368,7 @@ cmd_sendrmt(Display *dpy, char *cmd)
 		return CMD_STAT_NOCMD;
 	}
 
-	DBGPRN(errfp, "\n* REMOTE: Sender\n\n");
+	DBGPRN(DBG_RMT)(errfp, "\n* REMOTE: Sender\n\n");
 
 	/* Locate appropriate remote client window */
 	if ((win = cmd_getwin(dpy)) == 0)
@@ -2385,14 +2443,14 @@ cmd_propchg(Widget w, XtPointer client_data, XEvent *ev, Boolean *cont)
 	win = XtWindow(w);
 
 	if (EV_PROP_NOTIFY(ev, win, PropertyNewValue, cmd_req)) {
-		DBGPRN(errfp, "\n* REMOTE: Request received: ");
+		DBGPRN(DBG_RMT)(errfp, "\n* REMOTE: Request received: ");
 
 		if ((ret = XGetWindowProperty(dpy, XtWindow(w), cmd_req,
 				0, MAX_REQ_LEN, True, XA_STRING,
 				&type, &fmt, &n, &resid,
 				(unsigned char **) &reqstr)) != Success ||
 		    type != XA_STRING || reqstr == NULL) {
-			DBGPRN(errfp,
+			DBGPRN(DBG_RMT)(errfp,
 				"Cannot get window 0x%x req property.\n",
 				(int) XtWindow(w));
 		}
@@ -2433,17 +2491,19 @@ cmd_propchg(Widget w, XtPointer client_data, XEvent *ev, Boolean *cont)
 				&type, &fmt, &n, &resid,
 				(unsigned char **) &attstr)) != Success ||
 		    type != XA_STRING || attstr == NULL) {
-			DBGPRN(errfp,
+			DBGPRN(DBG_RMT)(errfp,
 				"Cannot get window 0x%x att property.\n",
 				(int) XtWindow(w));
 		}
 		else {
-			DBGPRN(errfp, "\n* REMOTE: Attach %s\n", attstr);
+			DBGPRN(DBG_RMT)(errfp,
+				"\n* REMOTE: Attach %s\n", attstr);
 		}
 	}
 	else if (EV_PROP_NOTIFY(ev, win, PropertyDelete, cmd_att)) {
 		if (attstr != NULL) {
-			DBGPRN(errfp, "\n* REMOTE: Detach %s\n", attstr);
+			DBGPRN(DBG_RMT)(errfp,
+				"\n* REMOTE: Detach %s\n", attstr);
 			XFree(attstr);
 			attstr = NULL;
 		}
