@@ -1,8 +1,8 @@
 /*
  *   xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-2000  Ti Kan
- *   E-mail: ti@amb.org
+ *   Copyright (C) 1993-2001  Ti Kan
+ *   E-mail: xmcd@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,16 +23,15 @@
 #define __HOTKEY_H__
 
 #ifndef LINT
-static char *_hotkey_h_ident_ = "@(#)hotkey.h	6.8 99/12/27";
+static char *_hotkey_h_ident_ = "@(#)hotkey.h	6.12 01/04/13";
 #endif
 
-#define MAX_TRANSLATIONS_SZ	4096
-
 /* Public functions */
 extern void	hotkey_init(void);
 extern void	hotkey_grabkeys(Widget);
 extern void	hotkey_ungrabkeys(Widget);
 extern void	hotkey_tooltip_mnemonic(Widget);
+extern bool_t	hotkey_ckkey(char *, char *);
 extern void	hotkey(Widget, XEvent *, String *, Cardinal *);
 
 #endif	/* __HOTKEY_H__ */
