@@ -1,8 +1,8 @@
 /*
  *   xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-2000  Ti Kan
- *   E-mail: ti@amb.org
+ *   Copyright (C) 1993-2001  Ti Kan
+ *   E-mail: xmcd@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #define __WIDGET_H__
 
 #ifndef LINT
-static char *_widget_h_ident_ = "@(#)widget.h	6.39 00/01/15";
+static char *_widget_h_ident_ = "@(#)widget.h	7.35 01/07/02";
 #endif
 
 /* Holder for all widgets */
@@ -47,7 +47,7 @@ typedef struct {
 		Widget	index_ind;		/* Index number indicator */
 		Widget	time_ind;		/* Time indicator */
 		Widget	rptcnt_ind;		/* Repeat count indicator */
-		Widget	dbmode_ind;		/* CD database indicator */
+		Widget	dbmode_ind;		/* CD information indicator */
 		Widget	progmode_ind;		/* Program mode indicator */
 		Widget	timemode_ind;		/* Time mode indicator */
 		Widget	playmode_ind;		/* Play mode indicator */
@@ -55,10 +55,11 @@ typedef struct {
 		Widget	ttitle_ind;		/* Track title indicator */
 		Widget	dbprog_btn;		/* CDDB/Program button */
 		Widget	options_btn;		/* Options button */
-		Widget	time_btn;		/* Time display button */
 		Widget	ab_btn;			/* A->B mode button */
 		Widget	sample_btn;		/* Sample mode button */
+		Widget	time_btn;		/* Time display button */
 		Widget	keypad_btn;		/* Keypad button */
+		Widget	wwwwarp_bar;		/* wwwWarp menu bar */
 		Widget	wwwwarp_btn;		/* wwwWarp button */
 		Widget	level_scale;		/* Volume control slider */
 		Widget	playpause_btn;		/* Play/Pause button */
@@ -148,17 +149,22 @@ typedef struct {
 		Widget	ok_btn;			/* OK button */
 	} options;
 
-	/* CD Database/Program window widgets */
+	/* CD Information/Program window widgets */
 	struct {
 		Widget	form;			/* Form container */
 		Widget	tottime_ind;		/* Total time indicator */
-		Widget	discid_ind;		/* Disc ID indicator */
+		Widget	inetoffln_btn;		/* Internet Offline button */
 		Widget	logo_lbl;		/* Logo label */
 		Widget	dlist_btn;		/* Disc list button */
-		Widget	dtitle_lbl;		/* Disc title label */
-		Widget	dtitle_txt;		/* Disc title text */
-		Widget	extd_lbl;		/* Disc ext descr label */
-		Widget	extd_btn;		/* Disc ext descr button */
+		Widget	artist_lbl;		/* Disc artist label */
+		Widget	artist_txt;		/* Disc artist text */
+		Widget	title_lbl;		/* Disc title label */
+		Widget	title_txt;		/* Disc title text */
+		Widget	fullname_btn;		/* Artist fullname button */
+		Widget	extd_lbl;		/* Disc label */
+		Widget	extd_btn;		/* Disc details button */
+		Widget	dcredits_btn;		/* Disc credits button */
+		Widget	segments_btn;		/* Segments button */
 		Widget	dbprog_sep1;		/* Separator bar */
 		Widget	trklist_lbl;		/* Track list label */
 		Widget	trklist_sw;		/* Track list scrolled window */
@@ -170,21 +176,21 @@ typedef struct {
 		Widget	trktime_btn;		/* Track time button */
 		Widget	ttitle_lbl;		/* Track title label */
 		Widget	ttitle_txt;		/* Track title text */
-		Widget	extt_lbl;		/* Track ext descr label */
-		Widget	extt_btn;		/* Track ext descr button */
-		Widget	pgm_lbl;		/* program label */
-		Widget	addpgm_btn;		/* Add button */
-		Widget	clrpgm_btn;		/* Clear button */
+		Widget	extt_lbl;		/* Track label */
+		Widget	extt_btn;		/* Track details button */
+		Widget	tcredits_btn;		/* Track credits button */
+		Widget	pgm_lbl;		/* Program label */
+		Widget	addpgm_btn;		/* Add program button */
+		Widget	clrpgm_btn;		/* Clear program button */
+		Widget	savepgm_btn;		/* Save program button */
 		Widget	pgmseq_lbl;		/* Program sequence label */
 		Widget	pgmseq_txt;		/* Program sequence text */
-		Widget	cddb_lbl;		/* CDDB label */
-		Widget	rmtdsbl_btn;		/* CDDB no remote button */
-		Widget	submit_btn;		/* CDDB submit button */
+		Widget	userreg_btn;		/* User registration button */
 		Widget	dbprog_sep2;		/* Separator bar */
-		Widget	savedb_btn;		/* Save to database button */
-		Widget	linkdb_btn;		/* Link database entry button */
-		Widget	loaddb_btn;		/* Load from database button */
-		Widget	cancel_btn;		/* Cancel button */
+		Widget	submit_btn;		/* CDDB submit button */
+		Widget	flush_btn;		/* CDDB flush cache button */
+		Widget	reload_btn;		/* Reload CD info button */
+		Widget	ok_btn;			/* OK button */
 	} dbprog;
 
 	/* Disc list window widgets */
@@ -208,83 +214,270 @@ typedef struct {
 		Widget	cancel_btn;		/* Cancel button */
 	} dlist;
 
-	/* Extended disc info window widgets */
+	/* Full name window widgets */
+	struct {
+		Widget	form;			/* Form container */
+		Widget	head_lbl;		/* Head label */
+		Widget	dispname_lbl;		/* display name label */
+		Widget	dispname_txt;		/* display name label */
+		Widget	autogen_btn;		/* auto-gen button */
+		Widget	lastname_lbl;		/* last name label */
+		Widget	lastname_txt;		/* last name label */
+		Widget	firstname_lbl;		/* first name label */
+		Widget	firstname_txt;		/* first name label */
+		Widget	the_btn;		/* "The" togglebutton */
+		Widget	the_txt;		/* "The" text */
+		Widget	fullname_sep;		/* Separator bar */
+		Widget	ok_btn;			/* OK button */
+	} fullname;
+
+	/* Disc details window widgets */
 	struct {
 		Widget	form;			/* Form container */
 		Widget	discno_lbl;		/* Disc number label */
-		Widget	disc_lbl;		/* Disc ext descr label */
-		Widget	disc_txt;		/* Disc ext descr text */
+		Widget	disc_lbl;		/* Disc artist/title label */
 		Widget	dbextd_sep;		/* Separator bar */
+		Widget	album_lbl;		/* Album label */
+		Widget	sorttitle_lbl;		/* Sort title label */
+		Widget	sorttitle_txt;		/* Sort title text */
+		Widget	the_btn;		/* The togglebutton */
+		Widget	the_txt;		/* The text */
+		Widget	year_lbl;		/* Year label */
+		Widget	year_txt;		/* Year text */
+		Widget	label_lbl;		/* Label label */
+		Widget	label_txt;		/* Label text */
+		Widget	comp_btn;		/* Compilation button */
+		Widget	genre_opt[2];		/* Genre option menu */
+		Widget	genre_menu[2];		/* Genre pulldown menu */
+		Widget	genre_lbl[2];		/* Genre menu label */
+		Widget	genre_sep[2];		/* Genre menu separator */
+		Widget	genre_none_btn[2];	/* Genre "none" button */
+		Widget	subgenre_opt[2];	/* Subgenre option menu */
+		Widget	subgenre_menu[2];	/* Subgenre pulldown menu */
+		Widget	subgenre_lbl[2];	/* Subgenre menu label */
+		Widget	subgenre_sep[2];	/* Subgenre menu separator */
+		Widget	subgenre_none_btn[2];	/* Subgenre "none" button */
+		Widget	dnum_lbl;		/* Disc number in set label */
+		Widget	dnum_txt;		/* Disc number in set text */
+		Widget	of_lbl;			/* "of" label */
+		Widget	tnum_txt;		/* Total number in set text */
+		Widget	region_lbl;		/* Region label */
+		Widget	region_txt;		/* Region text */
+		Widget	region_chg_btn;		/* Region change button */
+		Widget	dbextd_sep2;		/* Separator bar */
+		Widget	notes_lbl;		/* Disc notes label */
+		Widget	notes_txt;		/* Disc notes text */
+		Widget	discid_lbl;		/* Xmcd disc ID label */
+		Widget	discid_ind;		/* Xmcd disc ID ind label */
+		Widget	rev_lbl;		/* Revision label */
+		Widget	rev_ind;		/* Revision ind label */
+		Widget	cert_lbl;		/* Certifier label */
+		Widget	cert_ind;		/* Certifier ind label */
+		Widget	dbextd_sep3;		/* Separator bar */
 		Widget	ok_btn;			/* OK button */
-		Widget	clear_btn;		/* Clear button */
-		Widget	cancel_btn;		/* Cancel button */
 	} dbextd;
 
-	/* Extended track info window widgets */
+	/* Track details window widgets */
 	struct {
 		Widget	form;			/* Form container */
+		Widget	autotrk_btn;		/* Auto-track button */
 		Widget	prev_btn;		/* Prev track arrow button */
 		Widget	next_btn;		/* Next track arrow button */
 		Widget	trkno_lbl;		/* Track number label */
-		Widget	autotrk_btn;		/* Auto-track button */
-		Widget	trk_lbl;		/* Track ext descr label */
-		Widget	trk_txt;		/* Track ext descr text */
+		Widget	trk_lbl;		/* Track title label */
 		Widget	dbextt_sep;		/* Separator bar */
+		Widget	sorttitle_lbl;		/* Sort title label */
+		Widget	sorttitle_txt;		/* Sort title text */
+		Widget	the_btn;		/* The togglebutton */
+		Widget	the_txt;		/* The text */
+		Widget	artist_lbl;		/* Artist label */
+		Widget	artist_txt;		/* Artist text */
+		Widget	fullname_btn;		/* Artist full name button */
+		Widget	year_lbl;		/* Year label */
+		Widget	year_txt;		/* Year text */
+		Widget	label_lbl;		/* Label label */
+		Widget	label_txt;		/* Label text */
+		Widget	bpm_lbl;		/* BPM label */
+		Widget	bpm_txt;		/* BPM text */
+		Widget	genre_opt[2];		/* Genre option menu */
+		Widget	genre_menu[2];		/* Genre pulldown menu */
+		Widget	genre_lbl[2];		/* Genre menu label */
+		Widget	genre_sep[2];		/* Genre menu separator */
+		Widget	genre_none_btn[2];	/* Genre "none" button */
+		Widget	subgenre_opt[2];	/* Subgenre option menu */
+		Widget	subgenre_menu[2];	/* Subgenre pulldown menu */
+		Widget	subgenre_lbl[2];	/* Subgenre menu label */
+		Widget	subgenre_sep[2];	/* Subgenre menu separator */
+		Widget	subgenre_none_btn[2];	/* Subgenre "none" button */
+		Widget	dbextt_sep2;		/* Separator bar */
+		Widget	notes_lbl;		/* Track notes label */
+		Widget	notes_txt;		/* Track notes text */
+		Widget	isrc_lbl;		/* ISRC label */
+		Widget	isrc_ind;		/* ISRC indicator label */
+		Widget	dbextt_sep3;		/* Separator bar */
 		Widget	ok_btn;			/* OK button */
-		Widget	clear_btn;		/* Clear button */
-		Widget	cancel_btn;		/* Cancel button */
 	} dbextt;
 
-	/* Directory Selector window widgets */
+	/* Credits list/editor window */
 	struct {
 		Widget	form;			/* Form container */
-		Widget	dir_lbl;		/* Directory list label */
-		Widget	dir_list;		/* Directory list */
-		Widget	dirsel_sep;		/* Separator bar */
+		Widget	autotrk_btn;		/* Auto-track button */
+		Widget	prev_btn;		/* Prev track arrow button */
+		Widget	next_btn;		/* Next track arrow button */
+		Widget	disctrk_lbl;		/* Disc/track number label */
+		Widget	title_lbl;		/* Title label */
+		Widget	credlist_lbl;		/* Credit list label */
+		Widget	cred_list;		/* Credit list */
+		Widget	cred_sep1;		/* Separator */
+		Widget	crededit_lbl;		/* Credit editor label */
+		Widget	prirole_opt;		/* Role option menu */
+		Widget	prirole_menu;		/* Role pulldown menu */
+		Widget	prirole_lbl;		/* Role menu label */
+		Widget	prirole_sep;		/* Role menu separator */
+		Widget	prirole_none_btn;	/* Role "none" button */
+		Widget	subrole_opt;		/* Subrole option menu */
+		Widget	subrole_menu;		/* Subrole pulldown menu */
+		Widget	subrole_lbl;		/* Subrole menu label */
+		Widget	subrole_sep;		/* Subrole menu separator */
+		Widget	subrole_none_btn;	/* Subrole "none" button */
+		Widget	name_lbl;		/* Name label */
+		Widget	name_txt;		/* Name text */
+		Widget	fullname_btn;		/* Full name button */
+		Widget	notes_lbl;		/* Notes label */
+		Widget	notes_txt;		/* Notes text */
+		Widget	cred_sep2;		/* Separator */
+		Widget	add_btn;		/* Add button */
+		Widget	mod_btn;		/* Modify button */
+		Widget	del_btn;		/* Delete button */
 		Widget	ok_btn;			/* OK button */
-		Widget	cancel_btn;		/* Cancel button */
-	} dirsel;
+	} credits;
 
-	/* Link Selector window widgets */
+	/* Segments list/editor window */
 	struct {
 		Widget	form;			/* Form container */
-		Widget	link_lbl;		/* Directory list label */
-		Widget	link_list;		/* Directory list */
-		Widget	linksel_sep;		/* Separator bar */
+		Widget	discno_lbl;		/* Disc number label */
+		Widget	disc_lbl;		/* Disc artist/title label */
+		Widget	seglist_lbl;		/* Segment list label */
+		Widget	seg_list;		/* Segment list */
+		Widget	seg_sep1;		/* Separator bar */
+		Widget	segedit_lbl;		/* Segment editor label */
+		Widget	name_lbl;		/* Segment name label */
+		Widget	name_txt;		/* Segment name text */
+		Widget	start_lbl;		/* Start label */
+		Widget	end_lbl;		/* End label */
+		Widget	track_lbl;		/* Track label */
+		Widget	frame_lbl;		/* Frame label */
+		Widget	starttrk_txt;		/* Start track text */
+		Widget	startfrm_txt;		/* Start frame text */
+		Widget	endtrk_txt;		/* End track text */
+		Widget	endfrm_txt;		/* End frame text */
+		Widget	startptr_lbl;		/* Start pointer label */
+		Widget	endptr_lbl;		/* End pointer label */
+		Widget	playset_lbl;		/* Play-set label */
+		Widget	set_btn;		/* Set button */
+		Widget	playpaus_btn;		/* Play pause button */
+		Widget	stop_btn;		/* Stop button */
+		Widget	notes_lbl;		/* Segment notes label */
+		Widget	notes_txt;		/* Segment notes text */
+		Widget	segment_lbl;		/* Segment label */
+		Widget	credits_btn;		/* Credits button */
+		Widget	seg_sep2;		/* Separator bar */
+		Widget	add_btn;		/* Add button */
+		Widget	mod_btn;		/* Modify button */
+		Widget	del_btn;		/* Delete button */
 		Widget	ok_btn;			/* OK button */
-		Widget	cancel_btn;		/* Cancel button */
-	} linksel;
+	} segments;
 
-	/* wwwWarp window widgets */
+	/* Submit URL window */
 	struct {
 		Widget	form;			/* Form container */
-		Widget	wwwwarp_lbl;		/* pictorial logo label */
-		Widget	help_btn;		/* Help button */
-		Widget	sel_lbl;		/* Select one label */
-		Widget	sel_radbox;		/* Radio box */
-		Widget	site1_btn;		/* Web site 1 button */
-		Widget	site2_btn;		/* Web site 2 button */
-		Widget	site3_btn;		/* Web site 3 button */
-		Widget	discog_btn;		/* Local Discograph button */
-		Widget	srchweb_btn;		/* Search web button */
-		Widget	srchsite_menu;		/* Search sites pulldown menu */
-		Widget	srchsite_opt;		/* Search sites option menu */
-		Widget	srchsite_lbl;		/* Search sites menu label */
-		Widget	srchsite_sep;		/* Search sites lbl separator */
-		Widget	wwwwarp_sep;		/* Separator bar */
-		Widget	go_btn;			/* Go button */
-		Widget	cancel_btn;		/* Cancel button */
+		Widget	logo_lbl;		/* Logo label */
+		Widget	heading_lbl;		/* Heading label */
+		Widget	categ_lbl;		/* Category label */
+		Widget	categ_txt;		/* Category text */
+		Widget	name_lbl;		/* Name label */
+		Widget	name_txt;		/* Name text */
+		Widget	url_lbl;		/* URL label */
+		Widget	url_txt;		/* URL text */
+		Widget	desc_lbl;		/* Description label */
+		Widget	desc_txt;		/* Description text */
+		Widget	submiturl_sep;		/* Separator bar */
+		Widget	submit_btn;		/* Submit button */
+		Widget	ok_btn;			/* OK button */
+	} submiturl;
+
+	/* Region selector window widgets */
+	struct {
+		Widget	form;			/* Form container */
+		Widget	region_lbl;		/* Region list label */
+		Widget	region_list;		/* Region list */
+		Widget	region_sep;		/* Separator bar */
+		Widget	ok_btn;			/* OK button */
+	} regionsel;
+
+	/* Multiple match selector window widgets */
+	struct {
+		Widget	form;			/* Form container */
+		Widget	logo_lbl;		/* Match list logo label */
+		Widget	matchsel_lbl;		/* Match list label */
+		Widget	matchsel_list;		/* Match list */
+		Widget	matchsel_sep;		/* Separator bar */
+		Widget	ok_btn;			/* OK button */
+	} matchsel;
+
+	/* wwwWarp popup menu widgets */
+	struct {
+		Widget	menu;			/* Main popup menu */
+		Widget	genurls_menu;		/* General URLs menu */
+		Widget	discurls_menu;		/* Album-specific URLs menu */
 	} wwwwarp;
 
+	/* User registration window widgets */
+	struct {
+		Widget	form;			/* Form container */
+		Widget	logo_lbl;		/* Logo label */
+		Widget	caption_lbl;		/* Caption label */
+		Widget	handle_lbl;		/* Handle label */
+		Widget	handle_txt;		/* Handle text */
+		Widget	passwd_lbl;		/* Password label */
+		Widget	passwd_txt;		/* Password text */
+		Widget	vpasswd_lbl;		/* Verify password label */
+		Widget	vpasswd_txt;		/* Verify password text */
+		Widget	hint_lbl;		/* Password hint label */
+		Widget	hint_txt;		/* Password hint text */
+		Widget	gethint_btn;		/* Get hint button */
+		Widget	email_lbl;		/* Email addr label */
+		Widget	email_txt;		/* Email addr text */
+		Widget	region_lbl;		/* Region label */
+		Widget	region_txt;		/* Region text */
+		Widget	region_chg_btn;		/* Region change button */
+		Widget	postal_lbl;		/* Postal code label */
+		Widget	postal_txt;		/* Postal code text */
+		Widget	age_lbl;		/* Age label */
+		Widget	age_txt;		/* Age text */
+		Widget	gender_lbl;		/* Gender label */
+		Widget	gender_frm;		/* Gender frame */
+		Widget	gender_radbox;		/* Gender radio box */
+		Widget	male_btn;		/* Male button */
+		Widget	female_btn;		/* Female button */
+		Widget	unspec_btn;		/* Unspecified button */
+		Widget	allowmail_btn;		/* Allow mail toggle button */
+		Widget	allowstats_btn;		/* Allow stats toggle button */
+		Widget	userreg_sep;		/* separator bar */
+		Widget	ok_btn;			/* OK button */
+		Widget	priv_btn;		/* Privacy info button */
+		Widget	cancel_btn;		/* Cancel button */
+	} userreg;
+
 	/* Help window widgets */
 	struct {
 		Widget	form;			/* Form container */
 		Widget	topic_opt;		/* Topic option menu */
 		Widget	topic_menu;		/* Topic pulldown menu */
+		Widget	topic_lbl;		/* Topic menu label */
+		Widget	topic_sep;		/* Topic menu separator */
 		Widget	online_btn;		/* Online Help button */
-		Widget	menu_sep;		/* Menu Separator bar */
-		Widget	docs_lbl;		/* Xmcd Docs label */
-		Widget	menu_sep2;		/* Menu Separator bar */
+		Widget	topic_sep2;		/* Topic menu separator2 */
 		Widget	help_txt;		/* Help text */
 		Widget	help_sep;		/* Separator bar */
 		Widget	about_btn;		/* About button */
@@ -311,7 +504,6 @@ typedef struct {
 		Widget	fatal;			/* Fatal error popup */
 		Widget	confirm;		/* Confirm popup */
 		Widget	working;		/* Working popup */
-		Widget	dbmatch;		/* CDDB match query popup */
 		Widget	about;			/* About popup */
 	} dialog;
 
@@ -341,12 +533,12 @@ typedef struct {
 		Pixmap	dbprog_hlpixmap;	/* CDDB/Prog */
 		Pixmap	options_pixmap;		/* Options */
 		Pixmap	options_hlpixmap;	/* Options */
-		Pixmap	time_pixmap;		/* Time */
-		Pixmap	time_hlpixmap;		/* Time */
 		Pixmap	ab_pixmap;		/* A->B */
 		Pixmap	ab_hlpixmap;		/* A->B */
 		Pixmap	sample_pixmap;		/* Sample */
 		Pixmap	sample_hlpixmap;	/* Sample */
+		Pixmap	time_pixmap;		/* Time */
+		Pixmap	time_hlpixmap;		/* Time */
 		Pixmap	keypad_pixmap;		/* Keypad */
 		Pixmap	keypad_hlpixmap;	/* Keypad */
 		Pixmap	world_pixmap;		/* wwwWarp */
@@ -373,15 +565,33 @@ typedef struct {
 		Pixmap	ff_hlpixmap;		/* Search FF */
 	} main;
 
-	/* CD Database/Program window pixmaps */
+	/* CD Information/Program window pixmaps */
 	struct {
 		Pixmap	logo_pixmap;		/* CD Logo */
 	} dbprog;
 
-	/* wwwWarp window pixmaps */
+	/* CD Match window pixmaps */
 	struct {
-		Pixmap	wwwwarp_pixmap;		/* wwwWarp logo */
-	} wwwwarp;
+		Pixmap	logo_pixmap;		/* CDDB Logo */
+	} matchsel;
+
+	/* User registration window pixmaps */
+	struct {
+		Pixmap	logo_pixmap;		/* CDDB Logo */
+	} userreg;
+
+	/* Segments window pixmaps */
+	struct {
+		Pixmap	playpause_pixmap;	/* Play/Pause */
+		Pixmap	playpause_hlpixmap;	/* Play/Pause */
+		Pixmap	stop_pixmap;		/* Stop */
+		Pixmap	stop_hlpixmap;		/* Stop */
+	} segments;
+
+	/* SubmitURL window pixmaps */
+	struct {
+		Pixmap	logo_pixmap;		/* CDDB Logo */
+	} submiturl;
 
 	/* Dialog windows pixmaps */
 	struct {
