@@ -1,8 +1,8 @@
 /*
  *   xmcd - Motif(tm) CD Audio Player
  *
- *   Copyright (C) 1993-2000  Ti Kan
- *   E-mail: ti@amb.org
+ *   Copyright (C) 1993-2001  Ti Kan
+ *   E-mail: xmcd@amb.org
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
 #define __XMCD_H__
 
 #ifndef LINT
-static char *_xmcd_h_ident_ = "@(#)xmcd.h	6.14 99/12/27";
+static char *_xmcd_h_ident_ = "@(#)xmcd.h	6.17 01/04/09";
 #endif
 
 
@@ -36,6 +36,7 @@ static char *_xmcd_h_ident_ = "@(#)xmcd.h	6.14 99/12/27";
 #include <X11/keysym.h>
 #include <Xm/Xm.h>
 #include <Xm/ArrowB.h>
+#include <Xm/CascadeB.h>
 #include <Xm/Form.h>
 #include <Xm/Frame.h>
 #include <Xm/Label.h>
